/****************************************************************
+---------------------------------------------------------------+
|                      M A G N E T R O N                        |
|                      -----------------                        |
|                                                               |
|               Written by Steven De Toni Oct-2007              |
|                                                               |
|                This source code is Public Domain              |
+---------------------------------------------------------------+
*****************************************************************/
 
#include <stdlib.h>
#include "walker_objects.h"
#include "advanced_enemy.h"
#include "basic_enemy.h"
#include "level_design.h"
#include "background_flash.h"


AdvancedEnemy::~AdvancedEnemy()
{
}

AdvancedEnemy::AdvancedEnemy (ObjectType t)
{        
	dirChgTimer = direction = 0;
	pChkObjLock = NULL;

    switch (t)
    {
        case AET_BIG_BRIAN_BOSS:
            Create (GameBMResource[GR_ADV_BIG_BRIAN].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_BIG_BRIAN].hitPoints;
			m_Score      = GameBMResource[GR_ADV_BIG_BRIAN].score;
            m_FrameDelay = 2;
			dirChgTimer  = 0;
            break;

		case AET_IMPOSROBOT1:        
            Create (GameBMResource[GR_BASIC_IMPOSROBOT1].pResObj);
			m_HitPoints  = GameBMResource[GR_BASIC_IMPOSROBOT1].hitPoints;
			m_Score      = GameBMResource[GR_BASIC_IMPOSROBOT1].score;
            m_FrameDelay = 5;
			dirChgTimer  = 0;
            break;

		case AET_ORB_OF_POW:
        case AET_ORB_SMALL:
            Create (GameBMResource[GR_ADV_ORB_SMALL].pResObj);
			m_HitPoints = GameBMResource[GR_ADV_ORB_SMALL].hitPoints;
			m_Score     = GameBMResource[GR_ADV_ORB_SMALL].score;
            dirChgTimer = Dest_X = Dest_Y = -1;
			SDL_SetAlpha (m_Surface, SDL_SRCALPHA, 200);
            break;

        case AET_ORB_BIG:
            Create (GameBMResource[GR_ADV_ORB].pResObj);
			m_HitPoints = GameBMResource[GR_ADV_ORB].hitPoints;
			m_Score     = GameBMResource[GR_ADV_ORB].score;
            dirChgTimer = Dest_X = Dest_Y = -1;
			SDL_SetAlpha (m_Surface, SDL_SRCALPHA, 200);
            break;

		case AET_SHIPSHOOTER:
            Create (GameBMResource[GR_BASIC_ENEMY5].pResObj);
			m_HitPoints  = GameBMResource[GR_BASIC_ENEMY5].hitPoints;
			m_Score      = GameBMResource[GR_BASIC_ENEMY5].score;
			break;        

		case AET_BOSS_BRAIN:
			{
				Create (GameBMResource[GR_ADV_BRAIN1].pResObj);
				m_HitPoints  = GameBMResource[GR_ADV_BRAIN1].hitPoints;
				m_Score      = 20000;
				m_FrameDelay = 2;
				
				float deg = 360/10;
				for (int i = 0; i < 10; i++)
				{
					BasicEnemy* pRotatingObj = (BasicEnemy*)CreateNewObject (BET_BOSS_SHEILD_ROTATE, LLT_ENEMY);

					pRotatingObj->m_PosZ		 = (float)i * deg;
					pRotatingObj->pParentLocator = this;
					pRotatingObj->m_HitPoints    = 10;
					pRotatingObj->m_VelX	     = 80; // radius
					pRotatingObj->m_VelY         = 10; // rotate speed

					// Add a power up at random
					if ((rand() % 10) == 0)
					{
						pRotatingObj->m_DeathRelease         = WKO_POWFASTERFIRE;
						pRotatingObj->m_DeathRelAssocLstType = LLT_FRIENDS;
					}

					PEnemyList->AddSprite ((SDLSprite*)pRotatingObj);
				}
			}
			break;


        case AET_BULL_DROP:
            Create (GameBMResource[GR_ADV_BULLDROP].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_BULLDROP].hitPoints;
			m_Score      = GameBMResource[GR_ADV_BULLDROP].score;
            m_FrameDelay = 1;
            direction    = 0;
            m_Frame      = 0;
            break;

        case AET_LASER_VERT:
            Create (GameBMResource[GR_ADV_LASER_VERT].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_LASER_VERT].hitPoints;
			m_Score      = GameBMResource[GR_ADV_LASER_VERT].score;
            m_FrameDelay = 1;
            break;

        case AET_LASER_HORZ:
            Create (GameBMResource[GR_ADV_LASER_HORZ].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_LASER_HORZ].hitPoints;
			m_Score      = GameBMResource[GR_ADV_LASER_HORZ].score;
            m_FrameDelay = 1;
            break;

        case AET_BOSS1:
            Create (GameBMResource[GR_ADV_BOSS1].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_BOSS1].hitPoints;
			m_Score      = GameBMResource[GR_ADV_BOSS1].score;       
            m_FrameDelay = 1;
            break;

        case AET_ROBOTBULLY1:
            Create (GameBMResource[GR_ADV_ROBOTBULLY1].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_ROBOTBULLY1].hitPoints;
			m_Score      = GameBMResource[GR_ADV_ROBOTBULLY1].score;
            m_FrameDelay = 10;
            break;

        case AET_SPAWNER1:
            Create (GameBMResource[GR_ADV_SPAWNER].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_SPAWNER].hitPoints;
			m_Score      = GameBMResource[GR_ADV_SPAWNER].score;
            m_FrameDelay = 1;
            m_Frame      = rand() % m_NbTotFrame; 
            break;

        case AET_MUTANT1:
            Create (GameBMResource[GR_ADV_MUTANT1].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_MUTANT1].hitPoints;
			m_Score      = GameBMResource[GR_ADV_MUTANT1].score;
            m_FrameDelay = 20;
            break;

		case AET_BRAIN2:
        case AET_BRAIN1:
            Create (GameBMResource[GR_ADV_BRAIN1].pResObj);
			m_HitPoints  = GameBMResource[GR_ADV_BRAIN1].hitPoints;
			m_Score      = GameBMResource[GR_ADV_BRAIN1].score;
			m_FrameDelay = 2;
            break;

    }
    m_Type = t;
}

void AdvancedEnemy::LevelReset (void) 
{
    switch (m_Type)
    {
        case AET_LASER_VERT:
        case AET_LASER_HORZ:
            dirChgTimer = 0;
            break;

        case AET_ORB_BIG:
            dirChgTimer = Dest_X = Dest_Y = -1;
            break; 
			
		case AET_BULL_DROP:
			SetYFrameBank (0);
			direction = 0;
			break;
    }
}


SDL_bool AdvancedEnemy::SpriteHitPixel(SDLSprite* pSprite)
{
    switch (m_Type)
    {
        case AET_ORB_OF_POW:
            if (SDLSprite::SpriteHitPixel (pSprite) != SDL_FALSE)
            {
                // if hit by a player bullet, spawn a power up...
                switch ((ObjectType)pSprite->m_Type)
                {
                    case PLY_BULLET_LL:
                    case PLY_BULLET_LU:
                    case PLY_BULLET_UU:
                    case PLY_BULLET_RU:
                    case PLY_BULLET_RR:
                    case PLY_BULLET_RD:
                    case PLY_BULLET_DD:
                    case PLY_BULLET_LD:
			            {				
       			            SDLSprite* pNewObj = CreateNewObject ((ObjectType)(WKO_MAX + (rand() % 9)), LLT_FRIENDS);        

                            pNewObj->m_PosX   = m_PosX;
			                pNewObj->m_PosY   = m_PosY;    
				            pNewObj->m_VelX   = 0.5;
				            pNewObj->m_VelY   = 0.5;
                            SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pNewObj->m_PosX, (int)pNewObj->m_PosY);
        
                            // Create an implosion event to create a zombie
                            ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PFriendsList, pNewObj, 0};                                
                            PExpObjs->buildExplosionSplit (pNewObj, PC_EXPOUT, true, 0.7f, 60, 2, &event);
			            }
                }

                return SDL_TRUE;
            }
            break;

        case AET_ORB_SMALL:
            if (m_TimeToLive > 0) 
            {
                // Thus any point <x,y> lies on the circle if it satisfies this equation. 
                // If x*x + y*y - R*R < 0 then we know the point is inside the circle. 
                // Similarly, if x*x + y*y - R*R > 0 we know it is outside.
                int radius = (m_BlockWidth*2) + (pSprite->m_BlockWidth/2);

                int plyrX = (int)abs((m_PosX + (float)(m_BlockWidth/2)) - (pSprite->m_PosX + (float)(pSprite->m_BlockWidth/2)));
                int plyrY = (int)abs((m_PosY + (float)(m_BlockWidth/2)) - (pSprite->m_PosY + (float)(pSprite->m_BlockHeight/2)));

                if ( (((plyrX * plyrX) + (plyrY * plyrY)) - (radius * radius)) < 0 )
                {
                    return SDL_TRUE;
                }

                return SDL_FALSE;
            }
            if (SDLSprite::SpriteHitPixel (pSprite) != SDL_FALSE)
            {
                int i = 1;
                return SDL_TRUE;
            }
            break;

            
        case AET_ORB_BIG:
            if ((dirChgTimer == 0) && (pChkObjLock == pSprite))
            {
                dirChgTimer = -1;
                Mix_HaltChannel (SNDCHL_ENEMY_SHOOT2);
                return SDL_TRUE;
            }
            break;

        case AET_LASER_VERT:
            if ((dirChgTimer > 0) && (pSprite->m_IsBullet == SDL_FALSE))
            {
                int xFirePos = ((int)m_PosX)+(m_BlockWidth/2);

                // Do a quick bounds check to see if we have hit the player
                if ((xFirePos >= pSprite->m_PosX) && (xFirePos <= pSprite->m_PosX+pSprite->m_BlockWidth))
                {
				    // create a dynamic sprite work area.
				    SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, 1, pSprite->m_BlockHeight, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
        
                    SDL_Rect r = {0,0, 1, pSprite->m_BlockHeight};
                    SDL_FillRect (pBitmap, &r, 0xffffffff);

                    SDLSprite l;

                    l.Create  (pBitmap, r.w, r.h, 1, SDL_TRUE);
				    l.SetColorKey (0x00, 0x00, 0x00);

                    l.m_PosX = (float)xFirePos;
                    l.m_PosY = pSprite->m_PosY;
            
                    if (l.SpriteHitPixel(pSprite) != SDL_FALSE)
                    {
                        return SDL_TRUE;
                    }
                }
            }
            break;

        case AET_LASER_HORZ:
            if ((dirChgTimer > 0) && (pSprite->m_IsBullet == SDL_FALSE))
            {
                int yFirePos = ((int)m_PosY)+(m_BlockHeight/2);

                // Do a quick bounds check to see if we have hit the player
                if ((yFirePos >= pSprite->m_PosY) && (yFirePos <= pSprite->m_PosY+pSprite->m_BlockHeight))
                {
				    // create a dynamic sprite work area.
				    SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, pSprite->m_BlockWidth, 1, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
        
                    SDL_Rect r = {0,0, pSprite->m_BlockWidth, 1};
                    SDL_FillRect (pBitmap, &r, 0xffffffff);

                    SDLSprite l;

                    l.Create  (pBitmap, r.w, r.h, 1, SDL_TRUE);
				    l.SetColorKey (0x00, 0x00, 0x00);

                    l.m_PosX = pSprite->m_PosX;
                    l.m_PosY = (float)yFirePos;
            
                    if (l.SpriteHitPixel(pSprite) != SDL_FALSE)
                    {
                        return SDL_TRUE;
                    }
                }
            }
            break;

    }

    return SDLSprite::SpriteHitPixel (pSprite);
}

void AdvancedEnemy::Draw(SDL_Surface* lpSDLS, long int ScrnWorldX, long int ScrnWorldY)
{
    switch (m_Type)
    {
		case AET_ORB_OF_POW:
        case AET_ORB_SMALL:
            {
			    int angle;          
                if (m_TimeToLive == -1)
                {
				    for (int i = 0; i < 2; i++)
				    {
					    angle = (rand() % SINCOS_TABNUM);

					    SDL_Lightning (PScreen, 
								       (int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)2), 
								       (int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)2), 

								       (int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)((m_BlockWidth/2)  - (rand() % (m_BlockWidth/4)) - 1)), 
								       (int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)((m_BlockHeight/2) - (rand() % (m_BlockWidth/4)) - 1)), 
								       //(int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)(m_BlockWidth/2) ), 
								       //(int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)(m_BlockHeight/2) ), 
								       2, 2);
				    }				
                }
                else
                {

				    for (int i = 0; i < 7; i++)
				    {
					    angle = (rand() % SINCOS_TABNUM);

					    SDL_Lightning (PScreen, 
								       (int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)2), 
								       (int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)2), 

								       (int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)((m_BlockWidth*2)- (rand() % (m_BlockWidth/2))) ), 
								       (int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)((m_BlockHeight*2) - (rand() % (m_BlockWidth/2))) ), 
								       //(int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)(m_BlockWidth/2) ), 
								       //(int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)(m_BlockHeight/2) ), 
								       6, 6);                        
				    }				
                    return;
                }
            }
            break;

        case AET_ORB_BIG:
            {
			    int angle;
                
				SDLSprite* s = GameBMResource[GR_ADV_ORB2].pResObj;

				s->m_PosX = (m_PosX)+(float)(m_BlockWidth/2)  - (s->m_BlockWidth/2);
				s->m_PosY = (m_PosY)+(float)(m_BlockHeight/2) - (s->m_BlockHeight/2);

				s->Draw (lpSDLS);
                
				for (int i = 0; i < 2; i++)
				{
					angle = (rand() % SINCOS_TABNUM);

					SDL_Lightning (PScreen, 
								   (int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)5), 
								   (int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)5), 

								   (int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)((m_BlockWidth/2)  - (rand() % (m_BlockWidth/4)))), 
								   (int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)((m_BlockHeight/2) - (rand() % (m_BlockWidth/4)))), 
								   //(int)((int)m_PosX)+(m_BlockWidth/2)  + (int)(SIN[angle] * (float)(m_BlockWidth/2) ), 
								   //(int)((int)m_PosY)+(m_BlockHeight/2) + (int)(COS[angle] * (float)(m_BlockHeight/2) ), 
								   4, 4);

				}

                if ((Dest_X >= 0) && (Dest_Y >= 0))
                {

                    SDL_Lightning (PScreen, ((int)m_PosX)+(m_BlockWidth/2), ((int)m_PosY)+(m_BlockHeight/2),
                                   Dest_X, Dest_Y,     
                                   10, 10);

                }
				
            }
            break;

		case AET_LASER_VERT:
            if (dirChgTimer > 0)
            {
                Uint32 col   = SDL_MapRGB (PScreen->format, 0, 250 - rand() % 50, 250 - rand() % 150);

                int xFirePos = ((int)m_PosX)+(m_BlockWidth/2);

                SDL_line (PScreen, 
                          xFirePos, (int)GamePlayArea.y1,
                          xFirePos, (int)GamePlayArea.y2, col);
            }
			break;

        case AET_LASER_HORZ:
            if (dirChgTimer > 0)
            {
                Uint32 col   = SDL_MapRGB (PScreen->format, 250 - rand() % 50, 255 - rand() % 150, 0);
                int yFirePos = ((int)m_PosY)+(m_BlockHeight/2);

                SDL_line (PScreen, 
                          (int)GamePlayArea.x1, yFirePos, 
                          (int)GamePlayArea.x2, yFirePos, col);
            }
    }
    SDLSprite::Draw (lpSDLS, ScrnWorldX, ScrnWorldY);
}

void AdvancedEnemy::Move (SDLSprite* pPlayer)
{  
    switch (m_Type)
    {
		case AET_IMPOSROBOT1:
			if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
			{
				m_PosX = m_PosX - m_VelX;
				SetYFrameBank (1);
			}
			else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
			{
				m_PosX = m_PosX + m_VelX;
				SetYFrameBank (0);
			}

			if (m_PosY-m_VelY+(m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY - m_VelY;
			else if (m_PosY+m_VelY+(m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY + m_VelY;


			if ((dirChgTimer <= 0) && ((rand() % 75) == 0))
			{
				dirChgTimer = GameFPS.rate;
				pChkObjLock = CreateNewObject (DET_ROBOTELEC1, LLT_ELECTRODE);
				pChkObjLock->m_TimeToLive = GameFPS.rate; 
				PElectrodesList->AddSprite (pChkObjLock);
			}

			if (dirChgTimer > 0)
			{
				dirChgTimer--;
                bool safeObj = false;

			    // check if pChkObjLock is safe to use?		
			    for (SDLSprite* pNode = PElectrodesList->m_pList; pNode != NULL;)
			    {
				    if (pNode == pChkObjLock)
				    {
					    safeObj = true;
					    break;
				    }

                    if (pNode != NULL)
                        pNode = pNode->m_Next;
			    }
            
                if (safeObj != false)
                {               		
				    switch (GetYFrameBank())
				    {
					    case 1:
						    pChkObjLock->m_PosX  = m_PosX - (pChkObjLock->m_BlockWidth - 2);
						    pChkObjLock->m_PosY  = m_PosY - (pChkObjLock->m_BlockHeight/2)+3;
							pChkObjLock->m_Angle = -1.0f;
						    break;

					    default:
						    pChkObjLock->m_PosX  = m_PosX + (m_BlockWidth - 2);
						    pChkObjLock->m_PosY  = m_PosY - (pChkObjLock->m_BlockHeight/2)+3;
							pChkObjLock->m_Angle = 1.0f;
						    break;
				    }
                }
                else
                {
                    dirChgTimer = 0;
                    pChkObjLock = NULL;
                }
			}
			else
				pChkObjLock = NULL;

			m_VelX += m_VelInc;
			m_VelY += m_VelInc;
            AnimateSprite (GameFPS.totalframecount);
			break;

		case AET_BIG_BRIAN_BOSS:
			if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
			{
				m_PosX = m_PosX - m_VelX;
				SetYFrameBank (1);
			}
			else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
			{
				m_PosX = m_PosX + m_VelX;
				SetYFrameBank (0);
			}

			if (m_PosY-m_VelY+(m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY - m_VelY;
			else if (m_PosY+m_VelY+(m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY + m_VelY;

			if ((rand() % 100) == 50)
			{   
				DynamicEnemy* pShot = new DynamicEnemy ((ObjectType)(DET_HOMERSHOT1+(rand() % 2)));

				pShot->setUpHomer (m_PosX + (m_BlockWidth/2) + ((rand() % m_BlockWidth)  - (m_BlockWidth/2)), 
								   m_PosY + (m_BlockHeight/2)+ ((rand() % m_BlockHeight) - (m_BlockHeight/2)));
				PEnemyList->AddSprite (pShot);

				pShot->moveSpeed *= 2;
				
				PlaySnd (SNDT_SHOOT);
			}

			// Create a new brain!
			if ((rand() % 100) == 50)
            {
       			SDLSprite* pNewBrain = CreateNewObject (AET_BRAIN2, LLT_ENEMY);        

                pNewBrain->m_PosX   = m_PosX;
			    pNewBrain->m_PosY   = m_PosY;    
				pNewBrain->m_VelInc = m_VelInc;
				pNewBrain->m_VelX   = m_VelX * 2;
				pNewBrain->m_VelY   = m_VelY * 2;
                SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pNewBrain->m_PosX, (int)pNewBrain->m_PosY);
        
                // Create an implosion event to create a zombie
                ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PEnemyList, pNewBrain, 0};                                
                PExpObjs->buildExplosionSplit (pNewBrain, PC_EXPOUT, true, 0.7f, 60, 2, &event);
            }

			m_VelX += m_VelInc;
			m_VelY += m_VelInc;
			AnimateSprite (GameFPS.totalframecount);
			break;

		case AET_ORB_OF_POW:
            // Rebound off the walls of the arena
            if ((int)m_PosY+m_BlockHeight+m_VelY > GamePlayArea.y2)
                m_VelY = -m_VelY;
            if ((int)m_PosX+m_VelX < GamePlayArea.x1)
                m_VelX = -m_VelX;
            if ((int)m_PosY+m_VelY < GamePlayArea.y1)
                 m_VelY = -m_VelY;
            if ((int)m_PosX+m_BlockWidth+m_VelX > GamePlayArea.x2)
                m_VelX = -m_VelX;

			if ((rand() % 150) == 75)
			{				
       			SDLSprite* pNewObj = CreateNewObject ((ObjectType)(WKO_MAX + (rand() % 9)), LLT_FRIENDS);        

                pNewObj->m_PosX   = m_PosX;
			    pNewObj->m_PosY   = m_PosY;    
				pNewObj->m_VelX   = 0.5;
				pNewObj->m_VelY   = 0.5;
                SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pNewObj->m_PosX, (int)pNewObj->m_PosY);
        
                // Create an implosion event to create a zombie
                ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PFriendsList, pNewObj, 0};                                
                PExpObjs->buildExplosionSplit (pNewObj, PC_EXPOUT, true, 0.7f, 60, 2, &event);
			}

            m_PosX += m_VelX;
			m_PosY += m_VelY;
            m_Frame = GameFPS.currentticks % m_NbTotFrame;
			break;

        case AET_ORB_SMALL:            
            if (m_TimeToLive == -1)
            {
			    if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
				    m_PosX = m_PosX - m_VelX;
			    else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
				    m_PosX = m_PosX + m_VelX;

			    if (m_PosY-m_VelY+(m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				    m_PosY = m_PosY - m_VelY;
			    else if (m_PosY+m_VelY+(m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				    m_PosY = m_PosY + m_VelY;

			    m_VelX += m_VelInc;
			    m_VelY += m_VelInc;
            }
            break;

        case AET_ORB_BIG:
            // Rebound off the walls of the arena
            if ((int)m_PosY+m_BlockHeight+m_VelY > GamePlayArea.y2)
                m_VelY = -m_VelY;
            if ((int)m_PosX+m_VelX < GamePlayArea.x1)
                m_VelX = -m_VelX;
            if ((int)m_PosY+m_VelY < GamePlayArea.y1)
                 m_VelY = -m_VelY;
            if ((int)m_PosX+m_BlockWidth+m_VelX > GamePlayArea.x2)
                m_VelX = -m_VelX;


            // Calculate if player is within range for a shot...
            {
                // Thus any point <x,y> lies on the circle if it satisfies this equation. 
                // If x*x + y*y - R*R < 0 then we know the point is inside the circle. 
                // Similarly, if x*x + y*y - R*R > 0 we know it is outside.
                int radius = (m_BlockWidth*3);

                int plyrX = (int)abs((m_PosX + (float)(m_BlockWidth/2)) - (pPlayer->m_PosX + (float)(pPlayer->m_BlockWidth/2)));
                int plyrY = (int)abs((m_PosY + (float)(m_BlockWidth/2)) - (pPlayer->m_PosY + (float)(pPlayer->m_BlockHeight/2)));

                if ( (((plyrX * plyrX) + (plyrY * plyrY)) - (radius * radius)) < 0 )
                {
                    Dest_X = (int)(pPlayer->m_PosX + (float)(pPlayer->m_BlockWidth/2));
	                Dest_Y = (int)(pPlayer->m_PosY + (float)(pPlayer->m_BlockHeight/2));
                    
                    if (dirChgTimer > 0)
                    {
                        dirChgTimer--;

						((Player*)pPlayer)->add2Score (-10);

                        if (Mix_Playing (SNDCHL_ENEMY_SHOOT2) == 0)
                            Mix_PlayChannel (SNDCHL_ENEMY_SHOOT2, GameSNDResource[GRS_ELECTRIC1].pResSnd, -1);
                    }
                    else if (dirChgTimer < 0)
                    {
						pChkObjLock = pPlayer;
                        dirChgTimer = (int)((float)GameFPS.rate * 0.9f);
                        if (Mix_Playing (SNDCHL_ENEMY_SHOOT2) == 0)
                            Mix_PlayChannel (SNDCHL_ENEMY_SHOOT2, GameSNDResource[GRS_ELECTRIC1].pResSnd, -1);
                    }
                }
                else if (Dest_X >= 0)
                {
                    Dest_X = Dest_Y = -1;
                    dirChgTimer = -1;
                    Mix_HaltChannel  (SNDCHL_ENEMY_SHOOT2);
                }
            }

            m_PosX += m_VelX;
			m_PosY += m_VelY;
            m_Frame = GameFPS.currentticks % m_NbTotFrame;
            break;


		case AET_SHIPSHOOTER:
			if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
			{
				m_PosX = m_PosX - m_VelX;
				SetYFrameBank (1);
			}
			else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
			{
				m_PosX = m_PosX + m_VelX;
				SetYFrameBank (0);
			}

			if (m_PosY-m_VelY+(m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY - m_VelY;
			else if (m_PosY+m_VelY+(m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY + m_VelY;


			// Shoot at player...
			if ((m_PosY+m_BlockHeight/2 >= pPlayer->m_PosY) && 
			    (m_PosY+m_BlockHeight/2 <= pPlayer->m_PosY + pPlayer->m_BlockHeight) &&
				((GameFPS.totalframecount % 20) == 0))
			{
				SDLSprite* pShot = (SDLSprite*) new BasicEnemy (BET_SHIPSHOT1);
				
				switch (GetYFrameBank())
				{
					case 1:
						pShot->m_PosX = m_PosX + pShot->m_BlockWidth;		
						pShot->m_PosY = m_PosY + (m_BlockHeight/2) - (pShot->m_BlockHeight/2);		
						pShot->m_VelX = -4.0f;
						break;

					case 0:
					default:
						pShot->m_PosX = m_PosX + m_BlockWidth - pShot->m_BlockWidth;
						pShot->m_PosY = m_PosY + (m_BlockHeight/2) - (pShot->m_BlockHeight/2);		
						pShot->m_VelX = 4.0f;					
				}

				PEnemyList->AddSprite (pShot);
			}

			m_VelX += m_VelInc;
			m_VelY += m_VelInc;
			AnimateSprite (GameFPS.totalframecount);
			break;

		case AET_BOSS_BRAIN:
            // Rebound off the walls of the arena
            if ((int)m_PosY+m_BlockHeight+m_VelY > GamePlayArea.y2-m_BlockHeight)
                m_VelY = -m_VelY;
            if ((int)m_PosX+m_VelX < GamePlayArea.x1+m_BlockWidth)
                m_VelX = -m_VelX;
            if ((int)m_PosY+m_VelY < GamePlayArea.y1+m_BlockHeight)
                 m_VelY = -m_VelY;
            if ((int)m_PosX+m_BlockWidth+m_VelX > GamePlayArea.x2-m_BlockWidth)
                m_VelX = -m_VelX;

			// Create a zombie!
			if ((rand() % 40) == 5)
            {
       			SDLSprite* pZombie = CreateNewObject (WKO_ROBBYZOMBIE, LLT_ENEMY);        

                pZombie->m_PosX = m_PosX;
			    pZombie->m_PosY = m_PosY;    
                SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pZombie->m_PosX, (int)pZombie->m_PosY);
        
                // Create an implosion event to create a zombie
                ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PEnemyList, pZombie, 0};                                
                PExpObjs->buildExplosionSplit (pZombie, PC_EXPOUT, true, 0.7f, 60, 2, &event);
            }

			if ((rand() % 100) == 50)
			{   
				DynamicEnemy* pShot = new DynamicEnemy (DET_HOMERSHOT2);
                pShot->setUpHomer (m_PosX + (m_BlockWidth/2), m_PosY + (m_BlockHeight/2));
				PEnemyList->AddSprite (pShot);

				PlaySnd (SNDT_SHOOT);
			}

            m_PosX += m_VelX;
			m_PosY += m_VelY;	
			m_Frame = GameFPS.currentticks % m_NbTotFrame;
			break;

		case AET_LASER_VERT:
        case AET_LASER_HORZ:
			if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
				m_PosX = m_PosX - m_VelX;
			else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
				m_PosX = m_PosX + m_VelX;

			if (m_PosY-m_VelY+(m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY - m_VelY;
			else if (m_PosY+m_VelY+(m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY + m_VelY;

			m_VelX += m_VelInc;
			m_VelY += m_VelInc;
			    
            // *** Shoot at the player ***
            if (dirChgTimer > 0)    
                dirChgTimer--;  
            else if (GameFPS.totalframecount % (GameFPS.rate * 4) == 0)
            {
                dirChgTimer = GameFPS.rate;
                PlaySnd (SNDT_SHOOT);
            }
            
			AnimateSprite(GameFPS.totalframecount);
            break;

        case AET_BULL_DROP:
            // Move towards player
            if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX+(pPlayer->m_BlockWidth/2))
			{
                m_PosX = m_PosX - m_VelX;
				SetYFrameBank (1);
			}
            else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX+(pPlayer->m_BlockWidth/2))
			{
                m_PosX = m_PosX + m_VelX;
				SetYFrameBank (0);
			}

            if (direction == 0)
            {
                if ( ((pPlayer->m_PosX > m_PosX) && (pPlayer->m_PosX < m_PosX + m_BlockWidth)) && 
                     ((pPlayer->m_PosY+pPlayer->m_BlockHeight) > (m_PosY + m_BlockHeight)) )
                {
                    direction = 1;
                    m_Frame   = 1;
                }

                m_PosY -= m_VelY;
            }
            else if (direction == 1)
            {
                m_PosY += (m_VelY * 10);

                // Bouncy off bottom of the arena
                if ((int)m_PosY+m_BlockHeight > GamePlayArea.y2)
                {
                    direction = 0;
                    m_Frame   = 0;
                }
            }
            break;

        case AET_ROBOTBULLY1:
        case AET_BRAIN1:
		case AET_BRAIN2:
            {                   
                SDLSprite* pTarget = NULL;
                float      distX   = (float)PScreen->w;
                float      distY   = (float)PScreen->h;
                float      chkDistX, chkDistY;

                // Find the nearest friend to target first
	            for (SDLSprite* pNode = PFriendsList->m_pList; pNode != NULL;)
	            {                      
                    switch (pNode->m_Type)
                    {
						case WKO_WENCH:
                        case WKO_MAX:
                        case WKO_SAM:
                            break;

                        default:
							if (pNode != NULL)
								pNode = pNode->m_Next;
                            continue;
                    }

           			// Hit test brain to a friend   
                    switch (m_Type)
                    {      
                        case AET_ROBOTBULLY1:
        			        if (this->SpriteHitPixel(pNode) != SDL_FALSE)
                            {
                                KillFriendlySkullCrossbones (pNode);
				                PFriendsList->DelSprite(pNode);
								continue;
                            }
                            break;

						case AET_BRAIN2:
                        case AET_BRAIN1:
        			        if (this->SpriteHitPixel(pNode) != SDL_FALSE)
                            {
                                // Create an explosion event to create a zombie
                                PExpObjs->buildExplosionSplit (pNode, PC_VERT);
                                PExpObjs->buildExplosionSplit (pNode, PC_HORZ);

       				            SDLSprite* pZombie = CreateNewObject (WKO_ROBBYZOMBIE, LLT_ENEMY);        
	                            pZombie->m_PosX = pNode->m_PosX;
			                    pZombie->m_PosY = pNode->m_PosY;    
                                SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pZombie->m_PosX, (int)pZombie->m_PosY);
                        
                                // Create an implosion event to create a zombie
                                ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PEnemyList, pZombie, 0};                                
                                PExpObjs->buildExplosionSplit (pZombie, PC_EXPOUT, true, 0.7f, 60, 2, &event);

				                PFriendsList->DelSprite(pNode);
								continue;
                            }
                    }

                    chkDistX = abs(pNode->m_PosX - m_PosX);
                    chkDistY = abs(pNode->m_PosY - m_PosY);                    
                    if ((chkDistX+chkDistY < distX + distY))
                    {
                        distX = chkDistX;
                        distY = chkDistY;
                        pTarget = pNode;
                    }   
                    
                    if (pNode != NULL)
                        pNode = pNode->m_Next;
                }

                if (pTarget == NULL)
                   pTarget = (SDLSprite*)pPlayer;

                // Move to nearest target
                if (m_PosX-m_VelX+(m_BlockWidth/2) > pTarget->m_PosX+(pTarget->m_BlockWidth/2))
				{
                    m_PosX = m_PosX - m_VelX;
					SetYFrameBank (1);
				}
                else if (m_PosX+m_VelX+(m_BlockWidth/2) < pTarget->m_PosX+(pTarget->m_BlockWidth/2))
				{
                    m_PosX = m_PosX + m_VelX;
					SetYFrameBank (0);
				}

                if (m_PosY-m_VelY+(m_BlockHeight/2) > pTarget->m_PosY+(pTarget->m_BlockHeight/2))
                    m_PosY = m_PosY - m_VelY;
                else if (m_PosY+m_VelY+(m_BlockHeight/2) < pTarget->m_PosY+(pTarget->m_BlockHeight/2))
                    m_PosY = m_PosY + m_VelY;

				m_VelX += m_VelInc;
				m_VelY += m_VelInc;
			    
                // *** Shoot at the player ***
                switch (m_Type)
                {   
					case AET_BRAIN2:
			            if ((rand() % 100) == 50)
			            {   
				            DynamicEnemy* pShot = new DynamicEnemy ((ObjectType)(DET_HOMERSHOT1 + (GameFPS.currentticks % 2))); // pick a homer shot?
                            pShot->setUpHomer (m_PosX + (m_BlockWidth/2), m_PosY + (m_BlockHeight/2));
				            PEnemyList->AddSprite (pShot);
							PlaySnd (SNDT_SHOOT);
			            }

			            if ((rand() % 100) == 50)
			            {   
       						SDLSprite* pZombie = CreateNewObject (WKO_ROBBYZOMBIE, LLT_ENEMY);        

							pZombie->m_PosX = m_PosX;
							pZombie->m_PosY = m_PosY;    
							SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pZombie->m_PosX, (int)pZombie->m_PosY);
        
							// Create an implosion event to create a zombie
							ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PEnemyList, pZombie, 0};                                
							PExpObjs->buildExplosionSplit (pZombie, PC_EXPOUT, true, 0.7f, 60, 2, &event);
			            }
						break;

                    case AET_BRAIN1:
			            if ((rand() % 300) == 100)
			            {   
				            DynamicEnemy* pShot = new DynamicEnemy ((ObjectType)(DET_HOMERSHOT1 + (GameFPS.currentticks % 2))); // pick a homer shot?
                            pShot->setUpHomer (m_PosX + (m_BlockWidth/2), m_PosY + (m_BlockHeight/2));
				            PEnemyList->AddSprite (pShot);
							PlaySnd (SNDT_SHOOT);
			            }
                        break;
                }

				AnimateSprite(GameFPS.totalframecount);
            }
            break;

        case AET_BOSS1:
			if (m_PosX-m_VelX+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
				m_PosX = m_PosX - m_VelX;
			else if (m_PosX+m_VelX+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
				m_PosX = m_PosX + m_VelX;

			if (m_PosY-m_VelY+(m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY - m_VelY;
			else if (m_PosY+m_VelY+(m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
				m_PosY = m_PosY + m_VelY;

            // create new mutant blob
            if ((rand() % 50) == 20)
            {			
                SDLSprite* pBabyBoss = CreateNewObject (BET_BLOB1, LLT_ENEMY);
            
                pBabyBoss->m_PosX   = m_PosX + m_BlockWidth/2;
			    pBabyBoss->m_PosY   = m_PosY + m_BlockHeight/2;
                pBabyBoss->m_VelX   = m_VelX * 2;
                pBabyBoss->m_VelY   = m_VelY * 2;
                pBabyBoss->m_VelInc = m_VelInc;
                SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pBabyBoss->m_PosX, (int)pBabyBoss->m_PosY);
          
                // Create an implosion event to create a zombie
                ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PEnemyList, pBabyBoss, 0};
                PExpObjs->buildExplosionSplit (pBabyBoss, PC_EXPOUT, true, 0.7f, 60, 2, &event);
			}

            // Shot at player
			if ((rand() % 200) == 40)
			{   
				DynamicEnemy* pShot = new DynamicEnemy ((ObjectType)(DET_HOMERSHOT1 + (GameFPS.currentticks % 2))); // pick a homer shot?
                pShot->setUpHomer (m_PosX + (m_BlockWidth/2), m_PosY + (m_BlockHeight/2));
				PEnemyList->AddSprite (pShot);
				PlaySnd (SNDT_SHOOT);
			}

			m_VelX += m_VelInc;
			m_VelY += m_VelInc;
			break;            

        case AET_SPAWNER1:
            m_Frame = GameFPS.currentticks % m_NbTotFrame;
        case AET_MUTANT1:
			// Mutants, spawners move to corners of the screen and shoot at player ...
            if (dirChgTimer < 0)  // if spawning a mutant, stay put for a little time...
            {
                dirChgTimer++;
            }
			else if (dirChgTimer == 0)
			{
				int newDir = 0;
				
				while (newDir == direction)
				{
					newDir = rand() % 5;
				}

				// 0 = LeftTop, 1 = RightTop, 2 = leftBottom, 3 = RightBottom
				// Calculate path angle for obj
                int destPosX, destPosY;
				switch (newDir)
				{
					case 1:	// RightTop				
                        destPosY = GamePlayArea.y1 + (rand() % (GamePlayArea.h/30));
                        destPosX = GamePlayArea.x2 - (rand() % (GamePlayArea.w/30));
						break;

					case 2: // leftBottom
                        destPosY = GamePlayArea.y2 - (rand() % (GamePlayArea.h/30));
                        destPosX = GamePlayArea.x1 + (rand() % (GamePlayArea.w/30));
    					break;

					case 3: // RightBottom
                        destPosY = GamePlayArea.y2 - (rand() % (GamePlayArea.h/30));
                        destPosX = GamePlayArea.x2 - (rand() % (GamePlayArea.w/30));
						break;

					case 0: // LeftTop
					default:
                        destPosY = GamePlayArea.y1 + (rand() % (GamePlayArea.h/30));
                        destPosX = GamePlayArea.x1 + (rand() % (GamePlayArea.w/30));
						break;
				}

				// calculate direction and distance
                m_PosZ = (float)ATAN2(((float)destPosY) - m_PosY, ((float)destPosX)- m_PosX);

                // good old Pythagoras
                int distX    = abs(destPosX - (int)m_PosX);
                int distY    = abs(destPosY - (int)m_PosY);
                int distance = SQRTInt ((distX*distX) + (distY*distY));
				dirChgTimer  = (int) (((float)distance) / m_VelX); 
			}
            else if (dirChgTimer > 0) // if move to a destination...
            {
			    dirChgTimer--;

			    // *** calculate radial path of shot to player ***
			    if (m_PosZ < 0) 
			    {   // upper/negative half of circle 
				    m_PosX -= (COS[(int)((SINCOS_TABNUM/2) + m_PosZ)] * m_VelX);
				    m_PosY -= (SIN[(int)((SINCOS_TABNUM/2) + m_PosZ)] * m_VelX);
			    }
			    else 
			    {   // lower/positive half of circle
				    m_PosX += (COS[(int)m_PosZ] * m_VelX);
				    m_PosY += (SIN[(int)m_PosZ] * m_VelX);
			    }

			    m_VelX += m_VelInc;
			    m_VelY += m_VelInc;
            }
			
			// animate this obj
            AnimateSprite(GameFPS.totalframecount);

            // *** Shoot at the player ***
            switch (m_Type)
            {

                case AET_SPAWNER1:
                    // create new mutant
                    if ((rand() % 300) == 100)
                    {			
                        SDLSprite* pMutant = CreateNewObject (AET_MUTANT1, LLT_ENEMY);
                        
                        pMutant->m_PosX   = m_PosX;
			            pMutant->m_PosY   = m_PosY;
                        pMutant->m_VelX   = m_VelX;
                        pMutant->m_VelY   = m_VelY;
                        pMutant->m_VelInc = m_VelInc;
                        SNDPlayPan (SNDCHL_MUTANTSPAWN, GameSNDResource[GRS_MUTANTSPAWN].pResSnd, 0, (int)pMutant->m_PosX, (int)pMutant->m_PosY);
                      
                        // Create an implosion event to create a zombie
                        ParticalTrig event = {PARTEVNT_ADD2SPRITELIST, PEnemyList, pMutant, 0};
                        PExpObjs->buildExplosionSplit (pMutant, PC_EXPOUT, true, 0.7f, 60, 2, &event);

                        dirChgTimer = -60;
				    }
                    break;

                case AET_MUTANT1: 			
                    // create new shot
                    if ((rand() % 300) == 100)
                    {				
				        SDLSprite* pShot = CreateNewObject (BET_SHOT1, LLT_NULL);
				        pShot->m_PosX = m_PosX + (m_BlockWidth/2);
				        pShot->m_PosY = m_PosY + (m_BlockHeight/2);

				        // Calculate path to player
				        pShot->m_PosZ = (float) ATAN2((pPlayer->m_PosY + (pPlayer->m_BlockHeight/2)) - pShot->m_PosY, 
										              (pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))  - pShot->m_PosX);

                        pShot->m_VelR = 6.0;
				        PEnemyList->AddSprite (pShot);
			        }
            }
    }

    GameBoundryCheckUpdate (this);
}

bool AdvancedEnemy::MaimedHit (int shotType) 
{
	switch (m_Type)
	{             
		case AET_IMPOSROBOT1:
			if ((m_HitPoints <= 0) && (dirChgTimer > 0) && (pChkObjLock != NULL))
			{
				PElectrodesList->DelSprite (pChkObjLock);
			}
			break;

        case AET_ORB_SMALL:
            if (m_TimeToLive == -1)
            {
                m_TimeToLive = GameFPS.rate;   
                PExpObjs->buildExplosionSplit (this, PC_EXPOUT, false, 0.03f, GameFPS.rate);
                PlaySnd (SNDT_DEATH);
            }

            m_HitPoints  = 1;
            break;

        case AET_ORB_BIG:
            if (m_HitPoints <= 0)
            {
                Mix_HaltChannel (SNDCHL_ENEMY_SHOOT2);
                Dest_X = Dest_Y = -1;
            }
            break;

		case AET_BIG_BRIAN_BOSS:
		case AET_BOSS_BRAIN:
        case AET_BOSS1:
            GameFlasher.set (4, SDL_MapRGB(PScreen->format, 0xff, 0xff, 0xff));   
            break;

        // push back enemy if been shot at ..        
        case AET_LASER_HORZ:
		case AET_LASER_VERT:
        case AET_ROBOTBULLY1:
            switch (shotType)
            {
                case PLY_BULLET_LL:
                    m_PosX -= m_VelX * 20;
                    break;
                case PLY_BULLET_LU:
                    m_PosX -= m_VelX * 20;
                    m_PosY -= m_VelY * 20;
                    break;
                case PLY_BULLET_UU:
                    m_PosY -= m_VelY * 20;
                    break;
                case PLY_BULLET_RU:
                    m_PosX += m_VelX * 20;
                    m_PosY -= m_VelY * 20;
                    break;
                case PLY_BULLET_RR:
                    m_PosX += m_VelX * 20;
                    break;
                case PLY_BULLET_RD:
                    m_PosX += m_VelX * 20;
                    m_PosY += m_VelY * 20;
                    break;
                case PLY_BULLET_DD:
                    m_PosY += m_VelY * 20;
                    break;
                case PLY_BULLET_LD:
                    m_PosX -= m_VelX * 20;
                    m_PosY += m_VelY * 20;
                    break;
            }
            return true;
    }    
    return false;
}

void AdvancedEnemy::PlaySnd (int sndID)
{
	switch (sndID)
	{         
        case SNDT_SHOOT:
			switch (m_Type)
			{
                case AET_LASER_VERT:
                    SNDPlayPan (SNDCHL_ENEMY_SHOOT1, GameSNDResource[GRS_LASER1].pResSnd, 0, (int)m_PosX, (int)m_PosY);
                    break;

                case AET_LASER_HORZ:
                    SNDPlayPan (SNDCHL_ENEMY_SHOOT2, GameSNDResource[GRS_LASER1].pResSnd, 0, (int)m_PosX, (int)m_PosY);
                    break;

				case AET_BIG_BRIAN_BOSS:
				case AET_BOSS_BRAIN:
				case AET_BRAIN2:
				case AET_BRAIN1:
				case AET_BOSS1:
					SNDPlayPan (SNDCHL_ENEMY_SHOOT1, GameSNDResource[GRS_ENEMYSHOOT1].pResSnd, 0, (int)m_PosX, (int)m_PosY);					
					break;
            }
            break;

        case SNDT_DEATH:
			switch (m_Type)
			{			
                case AET_ORB_SMALL:
                    SNDPlayPan (SNDCHL_ENEMY_DEATH, GameSNDResource[GRS_EXPLODE2].pResSnd, 0, (int)m_PosX, (int)m_PosY);
                    break;

				case AET_BIG_BRIAN_BOSS:
				case AET_ORB_BIG:
				case AET_BOSS_BRAIN:
                case AET_BOSS1:					
					SNDPlayPan (SNDCHL_ENEMY_DEATH, GameSNDResource[GRS_EXPLODE1].pResSnd, 0, (int)m_PosX, (int)m_PosY);
					break;

				default:
                    SNDPlayPan (SNDCHL_ENEMY_HIT, GameSNDResource[GRS_ENEMYDEATH1].pResSnd, 0, (int)m_PosX, (int)m_PosY);
                    break;
            }
            break;

		case SNDT_HIT:            
            SNDPlayPan (SNDCHL_MAIMEHIT, GameSNDResource[GRS_MAIMEHIT].pResSnd, 0, (int)m_PosX, (int)m_PosY);             
			break;
	}
}

