/****************************************************************
+---------------------------------------------------------------+
|                      M A G N E T R O N                        |
|                      -----------------                        |
|                                                               |
|               Written by Steven De Toni Oct-2007              |
|                                                               |
|                This source code is Public Domain              |
+---------------------------------------------------------------+
*****************************************************************/
 
#include <stdlib.h>
#include "background_flash.h"
#include "level_design.h"

BackgroundFlash::BackgroundFlash ()
{
    steps    = 0;
    pos      = 0;
    startCol = 0;
	palIdx = 0;
}

void BackgroundFlash::set (int s, Uint32 sc)
{
    steps    = s;
    pos      = 0;
    startCol = sc;
    SDL_GetRGB(sc, PScreen->format, &r, &g, &b);
}

Uint32 BackgroundFlash::update (void)
{
    if (pos < steps)
    {
        pos++;
        return SDL_MapRGB (PScreen->format,  (r / steps)  * pos,  (g / steps)  * pos, (b / steps)  * pos);
    }        
    else
    {
        return 0x00000000;
    }
}

void BackgroundFlash::drawGameBorder (PlayAreaRect& playArea, int size, int palInc)
{        
    int      c, ci;
	Uint32   col;
	SDL_Rect r;
    int      xOrig = playArea.x1-1;
    int      yOrig = playArea.y1-1;
    int      xW    = playArea.w+1;
    int      yH    = playArea.h+1;

	for  (c = 0; c < size; c++)
	{
		ci = (palIdx+(2*c)) % 256;
        col = SDL_MapRGB (PScreen->format, SPECPAL[ci].r, SPECPAL[ci].g, SPECPAL[ci].b);

        // left top -> right top        
		r.x = xOrig-c;
        r.y = yOrig-c; 
        r.w = xW+(c*2), 
        r.h = 1;
		SDL_FillRect(PScreen, &r, col);

		// left top -> left bottom
		r.x = xOrig-c;
        r.y = yOrig-c; 
		r.w = 1;
        r.h = yH+(c*2);
		SDL_FillRect(PScreen, &r, col);


		// left bottom -> right bottom
		r.x = xOrig-c;
		r.y = yOrig+yH+c;
        r.w = xW+(c*2)+1;
        r.h = 1;
		SDL_FillRect(PScreen, &r, col);
			
		// right bottom -> right top
		r.x = xOrig+xW+c;
        r.y = yOrig-c;
        r.w = 1;
        r.h = yH+(c*2);
		SDL_FillRect(PScreen, &r, col);
	}

	palIdx = (palIdx+palInc) % 256;
}

Uint8 BackgroundFlash::fadeBetween(int start,int end, int proportion) 
{
	return (Uint8)(((end-start)*proportion)/128+start);
}

void BackgroundFlash::calWarpPalette (void) 
{
	int idx = 0;
	int i;
	
	switch ((rand() % 6))
	{
		case 0:
			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(0,255,i), 0, fadeBetween(255,0,i));
				idx++;
			}

			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(255,0,i), 0, fadeBetween(0,255,i));
				idx++;
			}
			break;
		case 1:
			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, 0, fadeBetween(0,255,i), fadeBetween(255,0,i));
				idx++;
			}

			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, 0, fadeBetween(255,0,i), fadeBetween(0,255,i));
				idx++;
			}
			break;

		case 2:
			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(255,0,i), fadeBetween(0,255,i), 0);
				idx++;
			}

			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(0,255,i), fadeBetween(255,0,i), 0);
				idx++;
			}
			break;

		case 3:
			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(255,0,i), fadeBetween(0,255,i), fadeBetween(0,255,i));
				idx++;
			}

			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(0,255,i), fadeBetween(255,0,i), fadeBetween(255,0,i));
				idx++;
			}
			break;

		case 4:
			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(255,0,i), fadeBetween(0,255,i), fadeBetween(255,0,i));
				idx++;
			}

			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(0,255,i), fadeBetween(255,0,i), fadeBetween(0,255,i));
				idx++;
			}
			break;

		case 5:
			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(0,255,i), fadeBetween(0,255,i), fadeBetween(255,0,i));
				idx++;
			}

			for(i = 0; i < 128; i++) 
			{
				warpPal[idx] = SDL_MapRGB (PScreen->format, fadeBetween(255,0,i), fadeBetween(255,0,i), fadeBetween(0,255,i));
				idx++;
			}
	}

}