/****************************************************************
+---------------------------------------------------------------+
|                      M A G N E T R O N                        |
|                      -----------------                        |
|                                                               |
|               Written by Steven De Toni Oct-2007              |
|                                                               |
|                This source code is Public Domain              |
+---------------------------------------------------------------+
*****************************************************************/
 
#include <stdlib.h>
#include <memory.h>
#include "dynamic_enemy.h"
#include "level_design.h"

DynamicEnemy::DynamicEnemy (ObjectType t, char* pBonusStr)
{        
    SDLSprite::Init();

    lightningRndRng = lightningSteps = movePntAimRng = movePntsMax = 0;
    moveSpeed = 0;
    movePnts  = NULL;

    switch (t)
    {
		case DET_ROBBYZOMBIE1:
			{
				SDLSprite::Init ();
				memset      (&workRect, 0, sizeof(workRect));

				m_HitPoints = GameBMResource[GR_DYN_HOMERSHOT1].hitPoints;
				m_Score     = GameBMResource[GR_DYN_HOMERSHOT1].score;

				lightningRndRng = 0;
				lightningSteps  = 1;
				movePntAimRng   = 1;
				moveSpeed       = 4.0f;
                movePntsMax     = 20;				
                movePnts        = new DM_POINTS[movePntsMax];		
				setUpHomer (0,0);
			}
			break;

		case DET_ROBOTELEC1:
			{
				// create a dynamic sprite work area.
				SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, 65, 30, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
            
				Create      (pBitmap, pBitmap->w, pBitmap->h, 1, SDL_TRUE);
				SetColorKey (0x00, 0x00, 0x00);
				memset      (&workRect, 0, sizeof(workRect));

				m_HitPoints     = MAX_INT;
				lightningRndRng = 5;
				lightningSteps  = 7;
			}
			break;

		case DET_HOMERSHOT2:
            {
				// create a dynamic sprite work area.
				SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, GameBMResource[GR_DYN_HOMERSHOT1].frameWidth, GameBMResource[GR_DYN_HOMERSHOT1].frameHeight, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
            
				Create      (pBitmap, GameBMResource[GR_DYN_HOMERSHOT1].frameWidth, GameBMResource[GR_DYN_HOMERSHOT1].frameHeight, GameBMResource[GR_DYN_HOMERSHOT1].frameNum, SDL_TRUE);
				SetColorKey (0x00, 0x00, 0x00);
				memset      (&workRect, 0, sizeof(workRect));

				m_HitPoints   = GameBMResource[GR_DYN_HOMERSHOT1].hitPoints;
				m_Score       = GameBMResource[GR_DYN_HOMERSHOT1].score;
                m_IsBullet    = SDL_TRUE;

				lightningRndRng = 3;
				lightningSteps  = 2;
				movePntAimRng   = 0;
				moveSpeed       = 2.0f;
                movePntsMax     = 10;
                movePnts        = new DM_POINTS[movePntsMax];                    
				setUpHomer (0,0);
            }
            break;

        case DET_HOMERSHOT1:
			{
				// create a dynamic sprite work area.
				SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, GameBMResource[GR_DYN_HOMERSHOT1].frameWidth, GameBMResource[GR_DYN_HOMERSHOT1].frameHeight, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
            
				Create      (pBitmap, GameBMResource[GR_DYN_HOMERSHOT1].frameWidth, GameBMResource[GR_DYN_HOMERSHOT1].frameHeight, GameBMResource[GR_DYN_HOMERSHOT1].frameNum, SDL_TRUE);
				SetColorKey (0x00, 0x00, 0x00);
				memset      (&workRect, 0, sizeof(workRect));

				m_HitPoints   = GameBMResource[GR_DYN_HOMERSHOT1].hitPoints;
				m_Score       = GameBMResource[GR_DYN_HOMERSHOT1].score;
                m_IsBullet    = SDL_TRUE;

				lightningRndRng = 3;
				lightningSteps  = 2;
				movePntAimRng   = 3;
				moveSpeed       = 2.0f;
                movePntsMax     = 7;
				m_IsBullet      = SDL_TRUE;
                movePnts        = new DM_POINTS[movePntsMax];
				setUpHomer (0,0);
			}
            break;

		case DET_BONUS:
			{
				if (pBonusStr == NULL)
					pBonusStr = "?!!?";

				int width  = SFont_TextWidth  (PGameFontInfoBonus[0], pBonusStr);
				int height = SFont_TextHeight (PGameFontInfoBonus[0]);

				// create a dynamic sprite work area.
				SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, (width * 4), height, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);
            
				Create      (pBitmap, width, height, 4, SDL_TRUE);
				SetColorKey (0x00, 0x00, 0x00);
                m_TimeToLive = GameFPS.rate/2;
				m_IsBullet   = SDL_TRUE;

				static SDL_Color cols[] = {  {0xff, 0x00, 0x00, 0x00},
											{0x00, 0xff, 0x00, 0x00},
											{0x00, 0x00, 0xff, 0x00},
											{0xff, 0xff, 0x00, 0x00},
											{0xff, 0x00, 0xff, 0x00},
											{0x00, 0xff, 0xff, 0x00}
											};											

				// Draw string in 4 different colours to dynamic surface
				int rnd = (rand() % 6);

				for (int idx = 0; idx < 4; idx++)
				{	
					rnd = (rnd + 1) % 6;

					SDL_ColPrintf (m_Surface, 
								   PGameFontInfoBonus[0], 
								   cols[rnd].r, 
								   cols[rnd].g, 
								   cols[rnd].b, 
								   2, idx * width, 0, 
								   pBonusStr);
				}
			}
			break;
    }
    m_Type = t;
}

DynamicEnemy::~DynamicEnemy()
{
    if (movePnts != NULL)
        delete[] movePnts;
}

void DynamicEnemy::setUpHomer (float x, float y)
{
    movePos.x     = x;
    movePos.y     = y;        
    movePos.velX  = 0;
    movePos.velY  = 0; 
    aimCntDwn     = 0;
    movePntIdx    = -1;
}

void DynamicEnemy::Draw(SDL_Surface* lpSDLS, long int ScrnWorldX, long int ScrnWorldY)
{        
    switch (m_Type)
    {
		case DET_BONUS:
			SetFrame(GameFPS.currentticks % m_NbTotFrame);
			SDLSprite::Draw (PScreen);
			break;

		case DET_ROBOTELEC1:
			// Blank out the prevous work bitmap.
			workRect.x = 0;
			workRect.y = 0;
			workRect.w = m_BlockWidth;
			workRect.h = m_BlockHeight;
			SDL_FillRect (m_Surface, &workRect, m_ColorKey);


			if (m_Angle > 0)
			{
				SDL_Lightning (m_Surface, 
							   0,					                       ((m_BlockHeight/2)+2) - (rand() % 4),  
							   m_BlockWidth - (rand() % (m_BlockWidth/4)), ((m_BlockHeight/2)+15) - (rand() % 30),   
							   lightningSteps, lightningRndRng);
			}
			else
			{
				SDL_Lightning (m_Surface, 
							   m_BlockWidth,				 ((m_BlockHeight/2)+2) - (rand() % 4),   
							   (rand() % (m_BlockWidth/4)),	 ((m_BlockHeight/2)+15) - (rand() % 30),  							   
							   lightningSteps, lightningRndRng);
			}

			SDLSprite::Draw (lpSDLS, ScrnWorldX, ScrnWorldY);
			break;

        case DET_HOMERSHOT1:
		case DET_HOMERSHOT2:
            //SDL_line (PScreen, (int)m_PosX, (int)m_PosY, (int)movePnts[movePntIdx].x, (int)movePnts[movePntIdx].y, 0xffffffff);
            //for (int idx = movePntIdx; idx > 0; idx--)
            //    SDL_line (lpSDLS, (int)movePnts[idx-1].x, (int)movePnts[idx-1].y,  (int)movePnts[idx].x, (int)movePnts[idx].y, 0xffffffff);

            // *** plot points ***
            
            // Blank out the prevous work bitmap, implement a dirty rectangle type of update.
            SDL_FillRect(m_Surface, &workRect, m_ColorKey);
            //SDL_FillRect(m_Surface, &workRect, SDL_MapRGB(m_Surface->format, 0xff, 0, 0));

            // determine max width and height;                
            float maxX     = movePos.x, maxY = movePos.y;
            int   overFlow = lightningSteps * lightningRndRng;
            int   idx;
                           
            // reposition on screen sprite to represent actual location.
            m_PosX = maxX;
            m_PosY = maxY;

            // Find min, max of drawing area.
            for (idx = 0; idx <= movePntIdx; idx++)
            {
                if (movePnts[idx].x <= m_PosX)
                    m_PosX = movePnts[idx].x-overFlow;
                if (movePnts[idx].x >= maxX)
                    maxX = movePnts[idx].x+overFlow;

                if (movePnts[idx].y <= m_PosY)
                    m_PosY = movePnts[idx].y-overFlow;
                if (movePnts[idx].y >= maxY)
                    maxY = movePnts[idx].y+overFlow;
            }

            SDL_Lightning (m_Surface, (int)(movePos.x - m_PosX), (int)(movePos.y - m_PosY), (int)(movePnts[movePntIdx].x - m_PosX), (int)(movePnts[movePntIdx].y - m_PosY), lightningSteps, lightningRndRng);
            for (idx = movePntIdx; idx > 0; idx--)
                SDL_Lightning (m_Surface, (int)(movePnts[idx-1].x - m_PosX), (int)(movePnts[idx-1].y - m_PosY),  (int)(movePnts[idx].x - m_PosX), (int)(movePnts[idx].y - m_PosY), lightningSteps, lightningRndRng);

            //SDL_line (m_Surface, (int)(movePos.x - m_PosX), (int)(movePos.y - m_PosY), (int)(movePnts[movePntIdx].x - m_PosX), (int)(movePnts[movePntIdx].y - m_PosY), 0xffffffff);
            //for (idx = movePntIdx; idx > 0; idx--)
            //    SDL_line (m_Surface, (int)(movePnts[idx-1].x - m_PosX), (int)(movePnts[idx-1].y - m_PosY),  (int)(movePnts[idx].x - m_PosX), (int)(movePnts[idx].y - m_PosY), 0xffffffff);

            // Draw the image ont the screen
            workRect.w = (Uint16)(maxX - m_PosX);
            workRect.h = (Uint16)(maxY - m_PosY);

            SDL_Rect dest = {(Sint16)m_PosX, 
                             (Sint16)m_PosY, 
                             workRect.w, workRect.h};

			m_BlockWidth  = workRect.w;
			m_BlockHeight = workRect.h;

            SDL_BlitSurface (m_Surface, &workRect, PScreen, &dest);
            break;
    } 
}

void DynamicEnemy::PlaySnd (int sndID)
{
	switch (sndID)
	{
		case SNDT_HIT:
            SNDPlayPan (SNDCHL_MAIMEHIT, GameSNDResource[GRS_MAIMEHIT].pResSnd, 0, (int)m_PosX, (int)m_PosY); 
            break;

        case SNDT_DEATH:
			switch (m_Type)
			{
				case DET_ROBBYZOMBIE1:
                case DET_HOMERSHOT1:  
		        case DET_HOMERSHOT2:
                    SNDPlayPan (SNDCHL_ENEMY_HIT, GameSNDResource[GRS_ENEMYHIT1].pResSnd, 0, (int)m_PosX, (int)m_PosY);
                    break;
			}
			break;
	}
}

void DynamicEnemy::Move (SDLSprite* pPlayer)
{
    switch (m_Type)
    {
		case DET_ROBOTELEC1:
			break;

		case DET_ROBBYZOMBIE1:
        case DET_HOMERSHOT1:  
		case DET_HOMERSHOT2:
			if (aimCntDwn <= 0)
			{
				float plyrX = pPlayer->m_PosX + (pPlayer->m_BlockWidth/2);
				float plyrY = pPlayer->m_PosY + (pPlayer->m_BlockHeight/2);

				switch (m_Type)
				{
					case DET_ROBBYZOMBIE1:
					case DET_HOMERSHOT1:
						if ((movePos.y > plyrY) && (movePos.velY <= 0) && (movePos.y-(movePntAimRng*2) > plyrY))
						{
							movePos.velX = 0;
							movePos.velY = -moveSpeed;
						}
						else if ((movePos.y < plyrY) && (movePos.velY >= 0) && (movePos.y+(movePntAimRng*2) < plyrY))
						{
							movePos.velX = 0;
							movePos.velY = moveSpeed;
						}
						else if (movePos.x > plyrX)
						{
							movePos.velX = -moveSpeed;
							movePos.velY = 0;
						}
						else if (movePos.x < plyrX) 
						{
							movePos.velX = moveSpeed;
							movePos.velY = 0;
						}
						break;

					case DET_HOMERSHOT2:
						if (movePos.y > plyrY )
							movePos.velY = -moveSpeed;
						if (movePos.y < plyrY)
							movePos.velY = moveSpeed;
						if (movePos.x > plyrX)
							movePos.velX = -moveSpeed;
						if (movePos.x < plyrX) 
							movePos.velX = moveSpeed;
						break;
				}

				// change the point, remove last expired point.
				movePntIdx++;
				if (movePntIdx >= movePntsMax)
				{
					// re-arrange points 
					for (int idx = 1; idx < movePntsMax; idx++)
						movePnts[idx-1] = movePnts[idx]; 

					movePntIdx--;
				}
        
				// Add new point direction.
				movePnts[movePntIdx].x    = movePos.x;
				movePnts[movePntIdx].y    = movePos.y;
				movePnts[movePntIdx].velX = movePos.velX;
				movePnts[movePntIdx].velY = movePos.velY;
    
				aimCntDwn = movePntAimRng;
			}
    
			// Front and end points
			if (movePntIdx > 0)
			{
				movePnts[0].x += movePnts[0].velX;
				movePnts[0].y += movePnts[0].velY;
			}

			movePos.x += movePos.velX;
			movePos.y += movePos.velY;
			aimCntDwn--;     
			break;
	}

    GameBoundryCheckUpdate (this);
}
