#include <stdlib.h>
#include <memory.h>
#include "level_design.h"
#include "advanced_enemy.h"
#include "basic_enemy.h"
#include "dynamic_enemy.h"
#include "walker_objects.h"
#include "background_flash.h"
#include "player.h"

PlayAreaRect GamePlayArea;
float	     GameLevelAdj = 1.0f;

bool GameBoundryCheckUpdate (SDLSprite* pObj)
{
    bool rtnCode = false; 

    if ((int)pObj->m_PosY+pObj->m_BlockHeight > GamePlayArea.y2)
    {
        pObj->m_PosY = (float)(GamePlayArea.y2 - pObj->m_BlockHeight);	
        rtnCode = true;
    }

    if ((int)pObj->m_PosX < GamePlayArea.x1)
    {
        pObj->m_PosX = (float)GamePlayArea.x1;
        rtnCode = true;
    }

    if ((int)pObj->m_PosY < GamePlayArea.y1)
    {
        pObj->m_PosY = (float)GamePlayArea.y1;
        rtnCode = true;
    }

    if ((int)pObj->m_PosX+pObj->m_BlockWidth > GamePlayArea.x2)
    {
        pObj->m_PosX = (float)(GamePlayArea.x2 - pObj->m_BlockWidth);
        rtnCode = true;
    }

    return rtnCode;
}

void SetupGameArena (void)
{
    // Setup Game boundries
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 25;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (GamePlayArea.w/2) + GamePlayArea.x1;
    GamePlayArea.yC = (GamePlayArea.h/2) + GamePlayArea.y1;
}

void PlotObjectInPlayArea  (SDLSprite* pObj)
{

    int x1 = GamePlayArea.xC - (GamePlayArea.w/6);
    int y1 = GamePlayArea.yC - (GamePlayArea.h/6);
    int x2 = GamePlayArea.xC + (GamePlayArea.w/6);
    int y2 = GamePlayArea.yC + (GamePlayArea.h/6);

    for (;;)
    {
        int x = (rand() % GamePlayArea.w) + GamePlayArea.x1;
        int y = (rand() % GamePlayArea.h) + GamePlayArea.y1;

        if (x+pObj->m_BlockWidth >= GamePlayArea.w + GamePlayArea.x1)
            x -= (x+pObj->m_BlockWidth) - (GamePlayArea.w + GamePlayArea.x1) - 1;

        if (y+pObj->m_BlockHeight >= GamePlayArea.h + GamePlayArea.y1)
            y -= (y+pObj->m_BlockHeight) - (GamePlayArea.h + GamePlayArea.y1) - 1;

        if ((x+pObj->m_BlockWidth  >= x1 && x <= x2) && 
            (y+pObj->m_BlockHeight >= y1 && y <= y2))
        {            
        }
        else
        {
			pObj->m_PosX = (float)x;
			pObj->m_PosY = (float)y;

			// Process object Exceptions
			switch ((ObjectType)pObj->m_Type)
			{
				case AET_BULL_DROP:
					if (pObj->m_PosX > GamePlayArea.xC)
					{
						if (pObj->m_PosX < x2)
							pObj->m_PosX = (float)x2;
					}
					else 
					{
						if (pObj->m_PosX+pObj->m_BlockWidth > x1)
							pObj->m_PosX = (float)(x1-pObj->m_BlockWidth);
					}
					break;

				case AET_BOSS_BRAIN:					
					if ((int)pObj->m_PosY + pObj->m_BlockHeight >= (GamePlayArea.y2 - pObj->m_BlockHeight))
						pObj->m_PosY = (float)(GamePlayArea.y2 - (pObj->m_BlockHeight * 2) - 2);
					if ((int)pObj->m_PosX <= (GamePlayArea.x1 + pObj->m_BlockWidth))
						pObj->m_PosX  = (float)(GamePlayArea.x1 + pObj->m_BlockWidth + 2);
					if ((int)pObj->m_PosY <= (GamePlayArea.y1 + pObj->m_BlockHeight))
						 pObj->m_PosY = (float)GamePlayArea.y1 + pObj->m_BlockHeight + 2;
					if ((int)pObj->m_PosX+pObj->m_BlockWidth >= (GamePlayArea.x2 - pObj->m_BlockWidth))
						pObj->m_PosX = (float)(GamePlayArea.x2 - (pObj->m_BlockWidth * 2) - 2);
					break;

				case DET_ROBOTELEC1:
				case BET_BOSS_SHEILD_ROTATE:
					pObj->m_PosY = pObj->m_PosX = -500;
					break;
			}
            return;
        }        
    }
}

void ListPlotObjectInPlayArea  (SDLSpriteList* pListObjs)
{
    for (SDLSprite* pNode = pListObjs->m_pList; pNode != NULL;)
    {
        PlotObjectInPlayArea  (pNode);
        pNode->LevelReset();

        if (pNode != NULL)
            pNode = pNode->m_Next;
    }
}

void ListAddObjectInPlay  (SDLSpriteList* pListObjs, SDLSprite* pObj)
{
    PlotObjectInPlayArea (pObj);
    pListObjs->AddSprite (pObj);
}

void JustDrawObjects (SDLSpriteList* pListObjs)
{
	for (SDLSprite* pNode = pListObjs->m_pList; pNode != NULL;)
    {        
        pNode->Draw(PScreen);

        if (pNode != NULL)
            pNode = pNode->m_Next;
    }
}

void DeleteBulletObjects (SDLSpriteList* pListObjs)
{
	for (SDLSprite* pNode = pListObjs->m_pList; pNode != NULL; )
    {
        if (pNode->m_IsBullet != SDL_FALSE)
        {
        	pListObjs->DelSprite(pNode);
            continue;
        }        

        if (pNode != NULL)
            pNode = pNode->m_Next;
    }
}

void AddList2Array (SDLSpriteList* pSrcObjs, SDLSprite* pDestArry[], int& idx, int maxSize)
{
	for (SDLSprite* pNode = pSrcObjs->m_pList; pNode != NULL;)
	{
		if (idx < maxSize)
		{
			pDestArry[idx] = pNode;
			idx++;
		}

        if (pNode != NULL)
            pNode = pNode->m_Next;
	}
}

void DoLevelWarp (Player& player)
{
	// Clear up screen and audio first ...
	while (PExpObjs->listNo > 0)
	{
		GamePlaySrnLock (player);	  
		// --------------------------

		PExpObjs->moveShowAll (PScreen);

		// --------------------------
		GamePlaySrnUnlock (player); 		
	}


	float xStep = 1;
	float yStep = ((float)(GamePlayArea.h) / (float)(GamePlayArea.w));
	float xPos  = 0;
	float yPos  = 0;
	PlayAreaRect r;
	
	r.xC = GamePlayArea.xC;
	r.yC = GamePlayArea.yC;
	r.x1 = r.xC;
	r.y1 = r.yC;
	r.x2 = r.xC;
	r.y2 = r.yC;
	r.w  = r.x2 - r.x1;
	r.h  = r.x2 - r.y1;

	Mix_PlayChannel(SNDCHL_GAMEMISC, GameSNDResource[GRS_ENDLEVEL].pResSnd, 0);
	
	int idx      = (GamePlayArea.w/2);
	int ci       = rand() % 256;
	float  palInc = 48.0f / (float)(GamePlayArea.w/2);
	float  palLowIdx  = 48.0f;
	float  palHighIdx = 48.0f + 127.0f;
	float  palIdx     = 0;
	bool   switcher   = false;

	GameFlasher.calWarpPalette ();

	while (idx >= 0)
	{
		GamePlaySrnLock (player, false);	  
		// --------------------------
		
		if (switcher != false)
			switcher = false;
		else
			switcher = true;

		for (int c = 0; (c < 6) && (idx >= 0); c++, idx--)
		{
			r.x1 = r.xC - (int)xPos;
			r.x2 = r.xC + (int)xPos;
			r.y1 = r.yC - (int)yPos;
			r.y2 = r.yC + (int)yPos;		

			if (switcher != true)
				palIdx = palLowIdx;
			else
				palIdx = palHighIdx;

			SDL_line (PScreen, r.x1, r.y1, r.x2, r.y1, GameFlasher.warpPal[(int)palIdx]);
			SDL_line (PScreen, r.x2, r.y1, r.x2, r.y2, GameFlasher.warpPal[(int)palIdx]);
			SDL_line (PScreen, r.x1, r.y2, r.x2, r.y2, GameFlasher.warpPal[(int)palIdx]);
			SDL_line (PScreen, r.x1, r.y1, r.x1, r.y2, GameFlasher.warpPal[(int)palIdx]);			
			r.w  = r.x2 - r.x1;
			r.h  = r.x2 - r.y1;

			xPos += xStep;
			yPos += yStep;

			ci++;
			palLowIdx  -= palInc;
			palHighIdx -= palInc;
		}
		
		// --------------------------
		GamePlaySrnUnlock (player); 		
	}

	idx = (GamePlayArea.w/2);

	r.xC = GamePlayArea.xC;
	r.yC = GamePlayArea.yC;
	r.x1 = r.xC;
	r.y1 = r.yC;
	r.x2 = r.xC;
	r.y2 = r.yC;
	r.w  = r.x2 - r.x1;
	r.h  = r.x2 - r.y1;
	xPos = 0;
	yPos = 0;

	while (idx >= 0)
	{
		GamePlaySrnLock (player, false);	  
		// --------------------------
		
		for (int c = 0; (c < 8) && (idx >= 0); c++, idx--)
		{
			r.x1 = r.xC - (int)xPos;
			r.x2 = r.xC + (int)xPos;
			r.y1 = r.yC - (int)yPos;
			r.y2 = r.yC + (int)yPos;						

			SDL_line (PScreen, r.x1, r.y1, r.x2, r.y1, 0);
			SDL_line (PScreen, r.x2, r.y1, r.x2, r.y2, 0);
			SDL_line (PScreen, r.x1, r.y2, r.x2, r.y2, 0);
			SDL_line (PScreen, r.x1, r.y1, r.x1, r.y2, 0);
			
			r.w  = r.x2 - r.x1;
			r.h  = r.x2 - r.y1;

			xPos += xStep;
			yPos += yStep;
		}
		xPos += (xStep * 4);
		yPos += (yStep * 4);
		
		// --------------------------
		GamePlaySrnUnlock (player); 		
	}

	idx = (GamePlayArea.w/2);

	r.xC = GamePlayArea.xC;
	r.yC = GamePlayArea.yC;
    r.x1 = r.xC;
	r.y1 = r.yC;
	r.x2 = r.xC;
	r.y2 = r.yC;
	r.w  = r.x2 - r.x1;
	r.h  = r.x2 - r.y1;
	xPos = 4;
	yPos = 4;

	while (idx >= 0)
	{
		GamePlaySrnLock (player, false);	  
		// --------------------------
		
		for (int c = 0; (c < 8) && (idx >= 0); c++, idx--)
		{
			r.x1 = r.xC - (int)xPos;
			r.x2 = r.xC + (int)xPos;
			r.y1 = r.yC - (int)yPos;
			r.y2 = r.yC + (int)yPos;						

			SDL_line (PScreen, r.x1, r.y1, r.x2, r.y1, 0);
			SDL_line (PScreen, r.x2, r.y1, r.x2, r.y2, 0);
			SDL_line (PScreen, r.x1, r.y2, r.x2, r.y2, 0);
			SDL_line (PScreen, r.x1, r.y1, r.x1, r.y2, 0);
			
			r.w  = r.x2 - r.x1;
			r.h  = r.x2 - r.y1;

			xPos += xStep;
			yPos += yStep;
		}
			xPos += (xStep * 4);
			yPos += (yStep * 4);
		
		// --------------------------
		GamePlaySrnUnlock (player); 		
	}
}

// -=========================================================================================-

void DoPauseScreen	(Player& player)
{
	char* s1 = "GAME PAUSED";
	char* s2 = "PRESS P TO CONTINUE";
	int   l1 = SFont_TextWidth  (PGameFontTitle, s1);
	int   l2 = SFont_TextWidth  (PGameFontTitle, s2);
	int   h  = SFont_TextHeight (PGameFontTitle);

	memset (GameKeyboard, false, sizeof(GameKeyboard));
    GamePlaying = false;

    
	while ((GameKeyboard[SDLK_ESCAPE] == false) && (GameKeyboard[SDLK_p] == false) && (MenuActivate () == false))
	{
		GamePlaySrnLock (player);	  
		// --------------------------

		JustDrawObjects (PEnemyList);
		JustDrawObjects (PFriendsList);	
		JustDrawObjects (PElectrodesList);	
		player.Draw(PScreen);
	    PExpObjs->showAll (PScreen);

		SDL_Printf (GamePlayArea.xC - (l1/2), GamePlayArea.yC-(GamePlayArea.h/4), s1);
		SDL_Printf (GamePlayArea.xC - (l2/2), GamePlayArea.yC-(GamePlayArea.h/4) + h + (h/2), s2);
	
		// --------------------------
		GamePlaySrnUnlock (player); 
		
	}
    GamePlaying = true;
	memset (GameKeyboard, false, sizeof(GameKeyboard));
}

void DoGameOver (Player& player)
{
	char* s1  = " G A M E ";
	char* s2  = "O V E R";
	int   l1  = SFont_TextWidth  (PGameFontTitle, s1);
    int   l2  = SFont_TextWidth  (PGameFontTitle, s2);
	int   h   = SFont_TextHeight (PGameFontTitle);
    int   wait;

    SDLSprite s;
    SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, l1, (h*2)+5, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
    SDL_Rect  r = {0, 0, pBitmap->w, pBitmap->h};
                
	s.Create       (pBitmap, pBitmap->w, pBitmap->h, 1, SDL_TRUE);
	s.SetColorKey  (0x00, 0x00, 0x00);
    SDL_FillRect   (pBitmap, &r, s.m_ColorKey);
    SDL_Printf     (pBitmap, PGameFontTitle, 0,0, s1);    
	SDL_Printf     (pBitmap, PGameFontTitle, (pBitmap->w/2) - (l2/2), h+5, s2);
    s.m_PosX = (float)(GamePlayArea.xC - (l1/2));
    s.m_PosY = (float)(GamePlayArea.yC - pBitmap->h);

    PExpObjs->buildExplosionSplit (&s, PC_EXPOUT, true, 0.1f, (GameFPS.rate*2), 1);    

    for (wait = 0; wait < (int)(GameFPS.rate*2); wait++)
	{
		GamePlaySrnLock (player);	        
		// --------------------------	        

		JustDrawObjects (PElectrodesList);	
		JustDrawObjects (PFriendsList);	
		JustDrawObjects (PEnemyList);
        
		PExpObjs->moveShowAll (PScreen);
			
		// --------------------------
		GamePlaySrnUnlock (player);
	}

    for (wait = 0; wait < (int)(GameFPS.rate*2); wait++)
	{
		GamePlaySrnLock (player);	        
		// --------------------------
		
		JustDrawObjects (PElectrodesList);	
		JustDrawObjects (PFriendsList);	
		JustDrawObjects (PEnemyList);        

        PExpObjs->moveShowAll (PScreen);

        s.Draw (PScreen);
			
		// --------------------------
		GamePlaySrnUnlock (player);
	}

    PExpObjs->buildExplosionSplit (&s, PC_EXPOUT, false, 0.5f, (GameFPS.rate*3), 1);
	Mix_HaltChannel (-1);
	Mix_PlayChannel (SNDCHL_PLAYER, GameSNDResource[GRS_SMARTBOMB1].pResSnd, 0);
    
    while (PExpObjs->listNo > 0)
	{
		GamePlaySrnLock (player);	        
		// --------------------------
		       
		JustDrawObjects (PElectrodesList);	
		JustDrawObjects (PFriendsList);	
		JustDrawObjects (PEnemyList);        

		PExpObjs->moveShowAll (PScreen);
			
		// --------------------------
		GamePlaySrnUnlock (player);
	}


    // Prune any bullets from lists
    DeleteBulletObjects (PEnemyList);
    DeleteBulletObjects (PPlayerBulletList);   
}

// -=========================================================================================-

void DoLevelStart (Player& player)
{	
#define DISPARRAYSIZE 1024 // max 1024 objects ... should be enough!

    Mix_Music*  pWarning         = NULL;
	SDLSprite** pInitDisplayList = new SDLSprite*[DISPARRAYSIZE];
	int		    initDispListIdx  = 0;	
	SDLSprite** pDisplayList	 = new SDLSprite*[DISPARRAYSIZE];	
	int		    dispListIdx	     = 0;	
	int			expldLen	     = 10;
	int			idx				 = 0;

    Mix_HaltChannel  (-1);

    // Position objects on screen ..
    ListPlotObjectInPlayArea  (PEnemyList);
    ListPlotObjectInPlayArea  (PFriendsList);
    ListPlotObjectInPlayArea  (PElectrodesList);
    player.levelReset();  
		
    memset (pInitDisplayList, 0, sizeof(SDLSprite*) * DISPARRAYSIZE);
	memset (pDisplayList,	  0, sizeof(SDLSprite*) * DISPARRAYSIZE);

	AddList2Array (PEnemyList,      pInitDisplayList, initDispListIdx, DISPARRAYSIZE);
	AddList2Array (PFriendsList,    pInitDisplayList, initDispListIdx, DISPARRAYSIZE);
	AddList2Array (PElectrodesList, pInitDisplayList, initDispListIdx, DISPARRAYSIZE);

	// -=================================================================-
	// Draw Game Objects
	// -=================================================================-
	// Loop the list and display the sprites
	for (bool endDisplay = false; endDisplay == false;)
	{   
        // Create an implosion event to create a object on screen
		if (idx < initDispListIdx)
		{
			ParticalTrig event = {PARTEVNT_ADD2SPRITEARRAY, pDisplayList, pInitDisplayList[idx], idx};

			// create an event to terminate this display when the last item is displayed.
			if (idx+1 >= initDispListIdx)
			{
				event.type  = PARTEVNT_SET_BOOL_DATA;
				event.pData = &endDisplay;
				event.data  = true;
			}

			PExpObjs->buildExplosionSplit (pInitDisplayList[idx], PCExplodeType (rand() % 2), true, 3.5, expldLen, 1, &event);
			dispListIdx++;
			idx++;
		}
		
		GamePlaySrnLock (player);

		if (GameKeyboard[SDLK_ESCAPE] != false)
			return;

		// --------------------------

		for (int refresh = 0; refresh < dispListIdx; refresh++)
		{
			if (pDisplayList[refresh] != NULL)
				pDisplayList[refresh]->Draw(PScreen); 
		}

		PExpObjs->moveShowAll (PScreen);
		
		// --------------------------            
		GamePlaySrnUnlock (player);	 		       
	}

	// -=================================================================-
	// Draw Player
	// -=================================================================-   
	player.SetYFrameBank (0);
    int frameCountDown = 70;
	static bool warningSaid = false;
                    
    // Chant robot warning to player...
    if (player.gameLevel == 8) 
    {                    
		if (warningSaid == false)
		{
			if (pWarning == NULL)
				pWarning = Mix_LoadMUS(_SPATH "sounds" _SEP "destroyhim.ogg");

			Mix_PlayMusic (pWarning, 1);
			frameCountDown = 140;
			warningSaid = true;
		}
		else
			player.PlaySnd(SNDT_INTRO);
    }
    else
	{
        player.PlaySnd(SNDT_INTRO);
		warningSaid = false;
	}

    PExpObjs->buildExplosionSplit (&player, PC_HORZ,   true,  1.0f, frameCountDown);
    PExpObjs->buildExplosionSplit (&player, PC_VERT,   true,  1.0f, frameCountDown);
    PExpObjs->buildExplosionSplit (&player, PC_HORZ,   true,  .5f,  frameCountDown);
    PExpObjs->buildExplosionSplit (&player, PC_VERT,   true,  .5f,  frameCountDown);
    PExpObjs->buildExplosionSplit (&player, PC_HORZ,   true,  3.5f, frameCountDown);
    PExpObjs->buildExplosionSplit (&player, PC_VERT,   true,  3.5f, frameCountDown);

    while (PExpObjs->listNo > 0)
    {
		GamePlaySrnLock (player);	 

		if (GameKeyboard[SDLK_ESCAPE] != false)
			return;		
		// --------------------------

        PExpObjs->moveShowAll (PScreen);

		JustDrawObjects (PElectrodesList);	
		JustDrawObjects (PFriendsList);	
		JustDrawObjects (PEnemyList);
		
		// --------------------------
		GamePlaySrnUnlock (player);        
    }

    if (pWarning != NULL)
        Mix_FreeMusic (pWarning);

	delete[] pDisplayList;
	delete[] pInitDisplayList;
}


void DoLevelDeath (Player& player)
{	
    // Create player death sequence.
	if (player.GetYFrameBank() == 2)
		player.SetYFrameBank (0);
	else if (player.GetYFrameBank() == 3)
		player.SetYFrameBank (1);

	PExpObjs->buildExplosionSplit (&player, PC_HORZ,   false,  .4f, 100);
	PExpObjs->buildExplosionSplit (&player, PC_VERT,   false,  .4f, 100);
	PExpObjs->buildExplosionSplit (&player, PC_HORZ,   false,  .2f, 110);
	PExpObjs->buildExplosionSplit (&player, PC_VERT,   false,  .2f, 110);
	PExpObjs->buildExplosionSplit (&player, PC_HORZ,   false,  .1f, 130);
	PExpObjs->buildExplosionSplit (&player, PC_VERT,   false,  .1f, 130);

	while (PExpObjs->listNo > 0)
	{
		GamePlaySrnLock (player);	        
		// --------------------------		

		JustDrawObjects (PElectrodesList);	
		JustDrawObjects (PFriendsList);	
		JustDrawObjects (PEnemyList); 

        PExpObjs->moveShowAll (PScreen);
				
		// --------------------------
		GamePlaySrnUnlock (player);
	}

    // Prune any bullets from lists    
    DeleteBulletObjects (PEnemyList);
    DeleteBulletObjects (PPlayerBulletList);   
	DeleteBulletObjects (PFriendsList);        

}

SDLSprite* CreateNewObject (ObjectType objType, LevelListType listType)
{
    SDLSprite* pNewObj = NULL;

    if ((objType) >= OBJT_BET && (objType < OBJT_AET))
    {
	    pNewObj = (SDLSprite*)new BasicEnemy(objType);              	
    }
    else if ((objType) >= OBJT_AET && (objType < OBJT_DET))
    {
        pNewObj = (SDLSprite*)new AdvancedEnemy(objType);   
    }
    else if ((objType) >= OBJT_DET && (objType < OBJT_WKO))
    {
        pNewObj = (SDLSprite*)new DynamicEnemy(objType);   
    }
    else if ((objType) >= OBJT_WKO && (objType < OBJT_PLY))
    {
        pNewObj = (SDLSprite*)new WalkerObject(objType);   
    }

	// Add random power-ups
	if (listType == LLT_ENEMY)
	{
		if (((rand() % 45) == 0) && (pNewObj->m_DeathRelease == 0))
		{
			pNewObj->m_DeathRelease			= (int)WKO_POWEXTRALIFE + (rand() % ((int)WKO_POWSHOCKBLASTER - (int)WKO_POWEXTRALIFE));
			pNewObj->m_DeathRelAssocLstType = LLT_FRIENDS;
		}
	}

    return pNewObj;
}

bool LoadLevel (int level, Player& player, LevelGoalType& levelGoal)
{
    // Remove all objects ...
	PExpObjs->deleteAll();
    PEnemyList->DeleteAll();
    PFriendsList->DeleteAll();
    PElectrodesList->DeleteAll();
    PPlayerBulletList->DeleteAll(); 

	levelGoal = PLevels[level].goalType;

    SetupGameArena ();

    for (int idx = 0; idx < LEVELS_MAXPARAM; idx++)
    {		
        if (PLevels[level].items[idx].num < 0)
            break;

        int powCount =  PLevels[level].items[idx].spawnNum;

        for (int newCnt = 0; newCnt < PLevels[level].items[idx].num; newCnt++)
        {
            SDLSprite* pNewObj = CreateNewObject (PLevels[level].items[idx].objType, PLevels[level].items[idx].listType);

            if (pNewObj != NULL)
            {
                // Set up inital speed data

				switch (PLevels[level].items[idx].objType)
				{
					case AET_ORB_OF_POW:
                    case AET_ORB_BIG:
					case WKO_RAIN1:
					case BET_ELECTRODE1:
						if ((rand() % 2) == 0)
							pNewObj->m_VelX	= PLevels[level].items[idx].VelX * GameLevelAdj;
						else
							pNewObj->m_VelX	= -PLevels[level].items[idx].VelX * GameLevelAdj;

						if ((rand() % 2) == 0)
							pNewObj->m_VelY	= PLevels[level].items[idx].VelY * GameLevelAdj;
						else
							pNewObj->m_VelY = -PLevels[level].items[idx].VelY * GameLevelAdj;
						pNewObj->m_VelInc = PLevels[level].items[idx].VelInc * GameLevelAdj;
						break;

					default:
						pNewObj->m_VelX   = PLevels[level].items[idx].VelX   * GameLevelAdj;
						pNewObj->m_VelY	  =	PLevels[level].items[idx].VelY   * GameLevelAdj;
						pNewObj->m_VelInc = PLevels[level].items[idx].VelInc * GameLevelAdj;
				}

				if (PLevels[level].items[idx].hitPoints > 0)
					pNewObj->m_HitPoints = PLevels[level].items[idx].hitPoints;

                // Add spawn pow, or enemy spawn for an object
                if (powCount > 0)
                {
                    powCount--;
                    pNewObj->m_DeathRelease         = PLevels[level].items[idx].spawnOnDeath;
					pNewObj->m_DeathRelAssocLstType = PLevels[level].items[idx].spawn2List;
                }

                // Add new obj to a list
                switch (PLevels[level].items[idx].listType)
                {
                    case LLT_ENEMY:
                        PEnemyList->AddSprite (pNewObj);
                        break;

                    case LLT_ELECTRODE:
                        PElectrodesList->AddSprite (pNewObj);
                        break;

                    case LLT_FRIENDS:
                        PFriendsList->AddSprite (pNewObj);
                        break;
                }
            }
        }
    }

    return true;
}
