#ifndef _LEVEL_DESIGN_
#define _LEVEL_DESIGN_

#include "objs.h"

// -================================================================-

typedef struct 
{
    int x1, y1; // left, top
    int x2, y2; // right, bottom
    int w,  h;  // width, height
    int xC, yC; // xCentre, yCentre;
    
} PlayAreaRect;

extern PlayAreaRect GamePlayArea;
extern float        GameLevelAdj;

// -================================================================-
enum LevelGoalType {LGT_KILLENEMY = 0, LGT_COLLECTFRIENDS};
enum LevelListType {LLT_NULL = 0, LLT_ENEMY, LLT_ELECTRODE, LLT_FRIENDS};

#define LEVELS_MAXPARAM 20
#define LEVELS_MAXLEVEL 30

typedef struct
{

    int           num;
	int		      hitPoints;
    ObjectType    objType;	
    LevelListType listType;
    float         VelX,    VelY;
	float		  VelInc;       // speed up velocity
    ObjectType    spawnOnDeath; // power ups
	LevelListType spawn2List;
    int           spawnNum;
	
} LevelItem;

typedef struct
{
	LevelGoalType goalType;
	char		  levelText[32];
	LevelItem	  items[LEVELS_MAXPARAM];  
} LevelInfo;


extern LevelInfo PLevels[];

// -================================================================-

inline bool MenuActivate (void)
{
    if ((GameKeyboard[SDLK_RETURN] != false) || 
	    (GameKeyboard[GameInfo.userKeys[USRKEY_FIREDOWN]]  != false) ||
	    (GameKeyboard[GameInfo.userKeys[USRKEY_FIREUP]]    != false) ||
	    (GameKeyboard[GameInfo.userKeys[USRKEY_FIRELEFT]]  != false) || 
	    (GameKeyboard[GameInfo.userKeys[USRKEY_FIRERIGHT]] != false))
	{
		return true;
	}
	return false;
}

// -================================================================-

bool GameBoundryCheckUpdate (SDLSprite* pObj);

SDLSprite* CreateNewObject (ObjectType objType, LevelListType listType);

bool LoadLevel (int level, Player& player, LevelGoalType& levelGoal);

void SetupGameArena			   (void);
void PlotObjectInPlayArea	   (SDLSprite* pObj);
void ListPlotObjectInPlayArea  (SDLSpriteList* pListObjs);
void ListAddObjectInPlay       (SDLSpriteList* pListObjs, SDLSprite* pObj);

void JustDrawObjects           (SDLSpriteList* pListObjs);

void DoLevelWarp			   (Player& player);
void DoGameOver                (Player& player);
void DoPauseScreen			   (Player& player);		
void DoLevelStart		       (Player& player);
void DoLevelDeath			   (Player& player);

#endif
