/***************************************************************************
                          SDLSpriteList.cpp  -  1.1.0
                             -------------------
    begin                : Tue Aug 15 2000
    copyright            : (C) 2000 by regis Quercioli
    email                : edorul@caramail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//////////////////////////////////////////////////////////////////////////////////
// based on :	$Source: /usr/cvsroot/cdx/src/cdx/cdxspritelist.cpp,v $
// 				$Author: MICHAELR $
// 				$Revision: 2.2 $
//
// Ported to SDL and modified by Regis Quercioli / edorul@caramail.com
//
// 1.1 : Add DelayStart as parametre in AddSprite function
//
//////////////////////////////////////////////////////////////////////////////////
#include "SDLSpriteList.h"

//////////////////////////////////////////////////////////////////////////////////
// Default constructor.
//////////////////////////////////////////////////////////////////////////////////
SDLSpriteList::SDLSpriteList(void)

{
	m_pList = NULL;
	m_nSprites = 0;
}

//////////////////////////////////////////////////////////////////////////////////
// Default contructor.
//////////////////////////////////////////////////////////////////////////////////
SDLSpriteList::~SDLSpriteList(void)
{
    DeleteAll();
}

//////////////////////////////////////////////////////////////////////////////////
// Adds a sprite to the sprite list. Only a pointer to a SDLTile object
// is required, however you can also set up the sprite as required.
//////////////////////////////////////////////////////////////////////////////////
SDLSprite* SDLSpriteList::AddSpriteVals (SDLSprite* pNode, int Type, float PosX, float PosY, float VelX, float VelY, int Frame)
{
	// Setup sprite
	pNode->SetType  (Type);
	pNode->SetPos   (PosX, PosY);
	pNode->SetVel   (VelX, VelY);
	pNode->SetFrame (Frame);
	
	// Add sprite too list
    return AddSprite (pNode);
}

//////////////////////////////////////////////////////////////////////////////////
// Adds a sprite to the sprite list. A pointer to an already existing SDLSprite
// is required.
//////////////////////////////////////////////////////////////////////////////////
SDLSprite* SDLSpriteList::AddSprite(SDLSprite* pNode)
{
    if (m_pList == NULL)
    {
        pNode->m_Next = NULL;
        pNode->m_Prev = NULL;
        m_pList = pNode;        
    }
    else
    {
        pNode->m_Next   = m_pList;
        pNode->m_Prev   = m_pList->m_Prev;
        m_pList->m_Prev = pNode;
        m_pList         = pNode;        
    }

	m_nSprites++;

	return pNode;
}

void SDLSpriteList::RemoveSprite(SDLSprite*& pNode)
{
    if (pNode == NULL)
        return;

    if (pNode->m_Next != NULL)
        pNode->m_Next->m_Prev = pNode->m_Prev;
    
    if (pNode->m_Prev != NULL)
        pNode->m_Prev->m_Next = pNode->m_Next;

    SDLSprite* pTmp = pNode->m_Next;

    if (m_pList == pNode)
        m_pList = pTmp;

    pNode = pTmp;
	m_nSprites--;
}

//////////////////////////////////////////////////////////////////////////////////
// Removes a sprite from the list.
//////////////////////////////////////////////////////////////////////////////////
void SDLSpriteList::DelSprite(SDLSprite*& pNode)
{
    if (pNode == NULL)
        return;

    if (pNode->m_Next != NULL)
        pNode->m_Next->m_Prev = pNode->m_Prev;
    
    if (pNode->m_Prev != NULL)
        pNode->m_Prev->m_Next = pNode->m_Next;

    SDLSprite* pTmp = pNode->m_Next;

    if (m_pList == pNode)
        m_pList = pTmp;

    delete pNode;

    pNode = pTmp;
	m_nSprites--;
}

//////////////////////////////////////////////////////////////////////////////////
// Draws all the sprites in the list to the surface pointed to by lpDDS.
//////////////////////////////////////////////////////////////////////////////////
void SDLSpriteList::Draw(SDL_Surface* lpSDLS, long int ScrnWorldX, long int ScrnWorldY)
{
	SDLSprite* Node;

	// Loop the list and draw the objects
	for(Node = m_pList; Node != NULL; Node = Node->m_Next)
	{
		Node->Draw(lpSDLS, ScrnWorldX, ScrnWorldY);
	}
}

