#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "menus.h"
#include "level_design.h"
#include "background_flash.h"

SFont_Font* PFontMenuTitle = NULL;
Mix_Music*  PBGMusic       = NULL;

#define NUMBER_OF_STARS 1020

//star struct
class STAR
{
public:
  float xpos, ypos;
  short zpos, speed;
  Uint8 color;

  STAR ()
  {
	  xpos  = ypos  = 0.0f;
	  zpos  = speed = 0;
	  color = 0;
  }
};

STAR stars[NUMBER_OF_STARS];

typedef struct
{
    int  x,y;
    char text[64];    
} MenuItem;


void stars_init (STAR* star, int i)
{
  /* randomly init stars, generate them around the center of the screen */
  
  star->xpos =  -10.0f + (20.0f * (rand()/(RAND_MAX+1.0f)));
  star->ypos =  -10.0f + (20.0f * (rand()/(RAND_MAX+1.0f)));
  
  star->xpos *= 3072.0f; /*change viewpoint */
  star->ypos *= 3072.0f;

  star->zpos  =  i;
  star->speed =  2 + (int)(2.0f * (rand()/(RAND_MAX+1.0f)));

  star->color = i >> 2; /*the closer to the viewer the brighter*/
}

void stars_initAll (void)
{
   // move and draw stars 
    for (int i = 0; i < NUMBER_OF_STARS; i++)
	{
		  stars[i].xpos =  -10.0f + (20.0f * (rand()/(RAND_MAX+1.0f)));
		  stars[i].ypos =  -10.0f + (20.0f * (rand()/(RAND_MAX+1.0f)));
  
		  stars[i].xpos *= 3072.0f; /*change viewpoint */
		  stars[i].ypos *= 3072.0f;

		  stars[i].zpos  =  rand() % 1024;
		  stars[i].speed =  2 + (int)(2.0f * (rand()/(RAND_MAX+1.0f)));

		  stars[i].color = ((int)stars[i].zpos % 256); /*the closer to the viewer the brighter*/	
	}
}

void stars_move (void)
{
	int i, tempx, tempy;  
	int centerx = PScreen->w/2;
	int centery = PScreen->h/2;

    // move and draw stars 
    for (i = 0; i < NUMBER_OF_STARS; i++)
	{
		stars[i].zpos -= stars[i].speed;

		if (stars[i].zpos <= 0)
		{
			stars_init(stars + i, i + 1);
		}

		/*compute 3D position*/
		tempx = (int)(stars[i].xpos / stars[i].zpos) + centerx;
		tempy = (int)(stars[i].ypos / stars[i].zpos) + centery;

		if (tempx < 0 || tempx > PScreen->w - 1 || tempy < 0 || tempy > PScreen->h - 1) /*check if a star leaves the screen*/
		{
			stars_init (stars + i, i + 1);
			continue;
		}

		DrawPixel(PScreen, tempx, tempy, stars[i].color, stars[i].color, stars[i].color);		
	}
}

void DisplayTitle (Player& player, bool init)
{
    static int  dispIdx         = -1;
	static int  lastDispIdx     = -1;
    static bool phraseShowCount = false;

    if (init != false)
    {
		static bool si = false;

		if (si == false)
		{
			stars_initAll ();
			si = true;
		}

        dispIdx         = -1;
	    lastDispIdx     = -1;
        phraseShowCount = false;
		
		
        GamePlaySrnLock (player);
        SDLSprite s;
    
        PExpObjs->deleteAll();

        for (int i = 0; i < (int)strlen(GAMETITLE); i++)
        {
            SDL_Printf (PScreen, PFontMenuTitle, 0, 0, "%c",GAMETITLE[i]);    
            s.Create(PScreen, 64, 64, 1, SDL_FALSE);   
            s.m_PosX = (float)((i * 64) + GAMETITLE_X);    
            s.m_PosY = GAMETITLE_Y;    

	        ParticalTrig event = {PARTEVNT_SET_INT_DATA, NULL, &dispIdx, i}; // have triger fire to display title in the DisplayMainMenu
            PExpObjs->buildExplosionSplit (&s, PC_VERT, true, 1.7f, 50 + (i * 15), 2, &event);

            Mix_PlayChannel (SNDCHL_ENEMY_HIT, GameSNDResource[GRS_MENUINTRO2].pResSnd, 0);                    
        }
    }


    PExpObjs->moveShowAll   (PScreen);

    // Display Title   
    for (int ti = 0; ti <= dispIdx; ti++)
        SDL_Printf (PScreen, PFontMenuTitle, ((ti * 64) + GAMETITLE_X), GAMETITLE_Y, "%c", GAMETITLE[ti]);        

    // Display phrase
    if (phraseShowCount != false)
    {
        SDL_Printf  (PScreen, PGameFontTitle, GAMETITLEPHRASE_X, GAMETITLEPHRASE_Y, GAMETITLEPHRASE);                   
    }

	// An explode objects has completed.
	if (lastDispIdx != dispIdx)
	{
		GameFlasher.set (7, SDL_MapRGB(PScreen->format, 0x7f, 0x7f, 0x7f));
		lastDispIdx = dispIdx;

        Mix_PlayChannel (SNDCHL_ENEMY_SHOOT1, GameSNDResource[GRS_MENUINTRO1].pResSnd, 0);        

        if (dispIdx == (int)(strlen(GAMETITLE)-1))
        {
            GameFlasher.set (20, SDL_MapRGB(PScreen->format, 0xff, 0xff, 0xff));            
            Mix_FadeInMusic(PBGMusic, -1, 9000);

	        int   l1  = SFont_TextWidth  (PGameFontTitle, GAMETITLEPHRASE);
	        int   h   = SFont_TextHeight (PGameFontTitle);                

            SDLSprite s;
            SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, l1, h, PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    
            SDL_Rect  r = {0, 0, pBitmap->w, pBitmap->h};

	        s.Create (pBitmap, pBitmap->w, pBitmap->h, 1, SDL_TRUE);
	        s.SetColorKey  (0x00, 0x00, 0x00);
            SDL_FillRect   (pBitmap, &r, s.m_ColorKey);
    
            SDL_Printf     (pBitmap, PGameFontTitle, 0,0, GAMETITLEPHRASE);                   
            s.m_PosX = GAMETITLEPHRASE_X;
            s.m_PosY = GAMETITLEPHRASE_Y;

            ParticalTrig event = {PARTEVNT_SET_BOOL_DATA, NULL, &phraseShowCount, true};
            PExpObjs->buildExplosionSplit (&s, PC_HORZ,   false, 0.1f, (GameFPS.rate*2), 1); 
            PExpObjs->buildExplosionSplit (&s, PC_EXPOUT, true,  0.08f, (GameFPS.rate*2), PARTBLKSIZE, &event);             
        }
	}

}

void ChangeCase (char* pSrc, bool upper)
{
    if (pSrc == NULL) return;

    if (upper == false)
    {
        while (*pSrc != '\0')
        {        
            if ((*pSrc >= 'A') && (*pSrc <= 'Z'))
                *pSrc = (char) (*pSrc - ('A' - 'a'));
            pSrc++;
        }
    }
    else
    {
        while (*pSrc != '\0')
        {        
            if ((*pSrc >= 'a') && (*pSrc <= 'z'))
                *pSrc = (char) (*pSrc + ('A' - 'a'));
            pSrc++;
        }
  
    }
}

void GetJoystickString (char* buff, int id)
{
	char joyMoveStrs[][16] = 
	{
		"JOY LEFT",
		"JOY UP",
		"JOY RIGHT",
		"JOY DOWN"
	};

	if (id < 0)
	{
		strcpy (buff, joyMoveStrs[abs(id)-1]);
		return;
	}

	sprintf (buff, "JOY BUT %d", id+1);
}


// -----------------------------------------------------------------------------

void DisplayJoystickMenu (Player& player)
{
#define KEYBRDMENU_NUM 9
	SDL_Event event;
    int       getJoyIdx = -1;
    bool      getJoyKey = false;

    int selectorMove  = 0;
    int selectorYDest = 0;
    int selItem       = 0;
    MenuItem menu[KEYBRDMENU_NUM] = 
    {
        {165, 110, "MOVE LEFT"},
        {165, 150, "MOVE UP"},
		{165, 190, "MOVE RIGHT"},
        {165, 230, "MOVE DOWN"},
        {165, 270, "SHOT LEFT"},
        {165, 310, "SHOT UP"},
        {165, 350, "SHOT RIGHT"},
        {165, 390, "SHOT DOWN"},
        {165, 430, "EXIT MAIN MENU"},
    };

    PlayAreaRect item = {menu[selItem].x - 15, menu[selItem].y - 10, item.x1 + 380, item.y1 + 35,  item.x2 - item.x1, item.y2 - item.y1, 0, 0};
    BackgroundFlash selector;
    selector.palIdx = 64;

    // Setup frame on whole
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while (GameKeyboard[SDLK_ESCAPE] == false)
	{
        if (getJoyKey != false)
        {
            while (SDL_PollEvent(&event))
            {
                if (event.type == SDL_QUIT)  
			        GameKeyboard[SDLK_ESCAPE] = (SDLKey)true;

				static bool joyBut[4] = {false,false,false, false}; // left, up, right, down

				switch (event.type)
				{			
					case SDL_JOYBUTTONDOWN:				
						if (event.jbutton.which != 0)
							break;

						GameInfo.userJoy[getJoyIdx] = event.jbutton.button;
						getJoyKey = false;
						break;
				
					case SDL_JOYAXISMOTION:
						if (event.jaxis.which != 0)
							break;

						switch (event.jaxis.axis)
						{
							case 0: // x
								switch (event.jaxis.value)
								{
									case -32768:
										GameInfo.userJoy[getJoyIdx] = USRJOY_LEFT;
										break;

									case 32767:
										GameInfo.userJoy[getJoyIdx] = USRJOY_RIGHT;
										break;

								}
								break;

							case 1: // y
								switch (event.jaxis.value)
								{
									case -32768:
										GameInfo.userJoy[getJoyIdx] = USRJOY_UP;
										break;

									case 32767:
										GameInfo.userJoy[getJoyIdx] = USRJOY_DOWN;
										break;
								}
								break;
						}
						getJoyKey = false;
				}

            }
        }

		GamePlaySrnLock (player);	  
        // --------------------------

		stars_move ();
		
        for (int i = 0; i < KEYBRDMENU_NUM; i++)
        {
            SDL_Printf (menu[i].x, menu[i].y, "%s", menu[i].text);

            if ( i < 8)
            {
                char buff[128];
                GetJoystickString (buff, GameInfo.userJoy[i]);
                ChangeCase(buff, true);
                if ((getJoyKey != false) && (getJoyIdx == i))
                {
                    SDL_Printf (menu[i].x+190, menu[i].y, ":");
                    PlayAreaRect item2 = item;

                    item2.x1 += 200;
                    item2.w   = item2.x2 - item2.x1;
                    item2.h   = item2.y2 - item2.y1;

                    selector.drawGameBorder (item2, 10, 1);
                }
                else
                    SDL_Printf (menu[i].x+190, menu[i].y, ": %s",buff);
            }
        }

        DisplayTitle (player, false);

        if (getJoyKey == false)
            selector.drawGameBorder (item, 10, 1);

        if (selectorMove == 0)
        {            
            if ((GameKeyboard[SDLK_DOWN] != false) && (selItem < KEYBRDMENU_NUM-1))
            {
                selectorMove  = 1;
                selectorYDest = menu[selItem+1].y-10;
                selItem++;
            }

            if ((GameKeyboard[SDLK_UP] != false) && (selItem > 0))
            {
                selectorMove  = -1;
                selectorYDest = menu[selItem-1].y-10;
                selItem--;
            }            
        }
        else if (selectorMove > 0)
        {
            if (item.y1 < selectorYDest)
            {
                item.y1 += 2;
                item.y2 += 2;
            }
            else
            {
                selectorMove = 0;
            }
        }
        else if (selectorMove < 0)
        {
            if (item.y1 > selectorYDest)
            {
                item.y1 -= 2;
                item.y2 -= 2;
            }
            else
            {
                selectorMove = 0;
            }
        }

        if (MenuActivate () != false)
        {
            if ((selItem >= 0) && (selItem < 8))
            {
                getJoyIdx = selItem;
                getJoyKey    = true;
                memset (GameKeyboard, false, sizeof(GameKeyboard));
            }

            switch (selItem)
            {
                case 8:         
                    memset (GameKeyboard, false, sizeof(GameKeyboard));
                    return;
            }
        }

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}
	memset (GameKeyboard, false, sizeof(GameKeyboard));
}

// -----------------------------------------------------------------------------

void DisplayKeyboardMenu (Player& player)
{
#define KEYBRDMENU_NUM 9
	SDL_Event event;
    int       getKeyIdx = -1;
    bool      getKey = false;

    int selectorMove  = 0;
    int selectorYDest = 0;
    int selItem       = 0;
    MenuItem menu[KEYBRDMENU_NUM] = 
    {
        {165, 110, "MOVE LEFT"},        
        {165, 150, "MOVE UP"},
		{165, 190, "MOVE RIGHT"},
        {165, 230, "MOVE DOWN"},
        {165, 270, "SHOT LEFT"},
        {165, 310, "SHOT UP"},
        {165, 350, "SHOT RIGHT"},
        {165, 390, "SHOT DOWN"},
        {165, 430, "EXIT MAIN MENU"},
    };

    PlayAreaRect item = {menu[selItem].x - 15, menu[selItem].y - 10, item.x1 + 325, item.y1 + 35,  item.x2 - item.x1, item.y2 - item.y1, 0, 0};
    BackgroundFlash selector;
    selector.palIdx = 64;

    // Setup frame on whole
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while (GameKeyboard[SDLK_ESCAPE] == false)
	{
        if (getKey != false)
        {
            while (SDL_PollEvent(&event))
            {
                if (event.type == SDL_QUIT)  
			        GameKeyboard[SDLK_ESCAPE] = (SDLKey)true;

		        if (event.type == SDL_KEYDOWN)
                {
                    if (event.key.keysym.sym != SDLK_ESCAPE)
                        GameInfo.userKeys[getKeyIdx] = event.key.keysym.sym;
                    getKey = false;
                }
            }
        }

		GamePlaySrnLock (player);	  
        // --------------------------

		stars_move ();
		
        for (int i = 0; i < KEYBRDMENU_NUM; i++)
        {
            SDL_Printf (menu[i].x, menu[i].y, "%s", menu[i].text);

            if ( i < 8)
            {
                char buff[128];
                strcpy (buff, SDL_GetKeyName(GameInfo.userKeys[i]));
                ChangeCase(buff, true);
                if ((getKey != false) && (getKeyIdx == i))
                {
                    SDL_Printf (menu[i].x+190, menu[i].y, ":");
                    PlayAreaRect item2 = item;

                    item2.x1 += 200;
                    item2.w   = item2.x2 - item2.x1;
                    item2.h   = item2.y2 - item2.y1;

                    selector.drawGameBorder (item2, 10, 1);
                }
                else
                    SDL_Printf (menu[i].x+190, menu[i].y, ": %s",buff);
            }
        }

        DisplayTitle (player, false);

        if (getKey == false)
            selector.drawGameBorder (item, 10, 1);

        if (selectorMove == 0)
        {            
            if ((GameKeyboard[SDLK_DOWN] != false) && (selItem < KEYBRDMENU_NUM-1))
            {
                selectorMove  = 1;
                selectorYDest = menu[selItem+1].y-10;
                selItem++;
            }

            if ((GameKeyboard[SDLK_UP] != false) && (selItem > 0))
            {
                selectorMove  = -1;
                selectorYDest = menu[selItem-1].y-10;
                selItem--;
            }            
        }
        else if (selectorMove > 0)
        {
            if (item.y1 < selectorYDest)
            {
                item.y1 += 2;
                item.y2 += 2;
            }
            else
            {
                selectorMove = 0;
            }
        }
        else if (selectorMove < 0)
        {
            if (item.y1 > selectorYDest)
            {
                item.y1 -= 2;
                item.y2 -= 2;
            }
            else
            {
                selectorMove = 0;
            }
        }

        if (MenuActivate () != false)
        {
            if ((selItem >= 0) && (selItem < 8))
            {
                getKeyIdx = selItem;
                getKey    = true;
                memset (GameKeyboard, false, sizeof(GameKeyboard));
            }

            switch (selItem)
            {
                case 8:         
                    memset (GameKeyboard, false, sizeof(GameKeyboard));
                    return;
            }
        }

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}
	memset (GameKeyboard, false, sizeof(GameKeyboard));
}

// -----------------------------------------------------------------------------

bool DisplayVideoMenu (Player& player)
{
#define VIDMENU_NUM 3
    int       getKeyIdx = -1;
    bool      getKey = false;
    bool      videoChange = false;

    int selectorMove  = 0;
    int selectorYDest = 0;
    int selItem       = 0;
    MenuItem menu[KEYBRDMENU_NUM] = 
    {
        {40, 150, "VIDEO MODE :    FULLSCREEN      WINDOW"},
        {40, 190, "VIDEO DEPTH:    32 BIT                    16 BIT"},
        {40, 250, "EXIT MAIN MENU"}, 
    };

    PlayAreaRect item = {menu[selItem].x - 15, menu[selItem].y - 10, item.x1 + 595, item.y1 + 35,  item.x2 - item.x1, item.y2 - item.y1, 0, 0};
    BackgroundFlash selector;
    selector.palIdx = 64;

    // Setup frame on whole
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while (GameKeyboard[SDLK_ESCAPE] == false)
	{
		GamePlaySrnLock (player);	

		stars_move ();
	
        if (getKey != false)
        {
		
			if (GameKeyboard[GameInfo.userKeys[USRKEY_LEFT]] != false)
            {
                switch (getKeyIdx)
                {
                    case 0:
                        GameInfo.fullScreen = true;
                        videoChange = true;
                        break;

                    case 1:
                        GameInfo.bitDepth = 32;
                        videoChange = true;
                        break;
                }
            }
            else if (GameKeyboard[GameInfo.userKeys[USRKEY_RIGHT]] != false)
            {
                switch (getKeyIdx)
                {
                    case 0:
                        GameInfo.fullScreen = false;
                        videoChange = true;
                        break;

                    case 1:
                        GameInfo.bitDepth = 16;
                        videoChange = true;
                        break;
                }
            }                    
			else if (MenuActivate () != false)
			{
				memset (GameKeyboard, false, sizeof(GameKeyboard));
                getKey = false;
			}
        }
	
        // --------------------------
		
        for (int i = 0; i < VIDMENU_NUM; i++)
        {
            SDL_Printf (menu[i].x, menu[i].y, "%s", menu[i].text);

            if ( i < VIDMENU_NUM)
            {
                if ((getKey != false) && (getKeyIdx == i))
                {
                    PlayAreaRect item2 = item;

                    switch (getKeyIdx)
                    {
                        case 0:
                            if (GameInfo.fullScreen == false)
                            {
                                item2.x1 += 445;
                            }
                            else
                            {
                                item2.x1 += 225;
                                item2.x2 -= 175;
                            }
                            break;

                        case 1:
                            if (GameInfo.bitDepth == 16)
                            {
                                item2.x1 += 445;
                            }
                            else
                            {
                                item2.x1 += 225;
                                item2.x2 -= 175;
                            }                            
                            break;
                    }

                    item2.w   = item2.x2 - item2.x1;
                    item2.h   = item2.y2 - item2.y1;

                    selector.drawGameBorder (item2, 10, 1);
                }
                
                // draw line identifing current selection option.
                if ((GameInfo.fullScreen == false) && (i == 0))
                {
                    SDL_line(PScreen,menu[0].x+440, menu[i].y+20, menu[i].x+570, menu[i].y+20, SDL_MapRGB (PScreen->format, SPECPAL[GameFlasher.palIdx].r, SPECPAL[GameFlasher.palIdx].g, SPECPAL[GameFlasher.palIdx].b));
                }
                else if ((GameInfo.fullScreen != false) && (i == 0))
                {
                    SDL_line(PScreen,menu[0].x+220, menu[i].y+20, menu[i].x+395, menu[i].y+20, SDL_MapRGB (PScreen->format, SPECPAL[GameFlasher.palIdx].r, SPECPAL[GameFlasher.palIdx].g, SPECPAL[GameFlasher.palIdx].b));
                }
                if ((GameInfo.bitDepth == 16) && (i == 1))
                {
                    SDL_line(PScreen,menu[0].x+440, menu[i].y+20, menu[i].x+570, menu[i].y+20, SDL_MapRGB (PScreen->format, SPECPAL[GameFlasher.palIdx].r, SPECPAL[GameFlasher.palIdx].g, SPECPAL[GameFlasher.palIdx].b));
                }
                else if ((GameInfo.bitDepth == 32) && (i == 1))
                {
                    SDL_line(PScreen,menu[0].x+220, menu[i].y+20, menu[i].x+395, menu[i].y+20, SDL_MapRGB (PScreen->format, SPECPAL[GameFlasher.palIdx].r, SPECPAL[GameFlasher.palIdx].g, SPECPAL[GameFlasher.palIdx].b));
                }

            }
        }

        DisplayTitle (player, false);

        if (getKey == false)
            selector.drawGameBorder (item, 10, 1);

        if ((selectorMove == 0) && (getKey == false))
        {            
            if ((GameKeyboard[SDLK_DOWN] != false) && (selItem < VIDMENU_NUM-1))
            {
                selectorMove  = 1;
                selectorYDest = menu[selItem+1].y-10;
                selItem++;
            }

            if ((GameKeyboard[SDLK_UP] != false) && (selItem > 0))
            {
                selectorMove  = -1;
                selectorYDest = menu[selItem-1].y-10;
                selItem--;
            }            
        }
        else if (selectorMove > 0)
        {
            if (item.y1 < selectorYDest)
            {
                item.y1 += 2;
                item.y2 += 2;
            }
            else
            {
                selectorMove = 0;
            }
        }
        else if (selectorMove < 0)
        {
            if (item.y1 > selectorYDest)
            {
                item.y1 -= 2;
                item.y2 -= 2;
            }
            else
            {
                selectorMove = 0;
            }
        }

        if (MenuActivate () != false)
        {
            if ((selItem >= 0) && (selItem < VIDMENU_NUM))
            {
                getKeyIdx = selItem;
                getKey    = true;
                memset (GameKeyboard, false, sizeof(GameKeyboard));
            }

            switch (selItem)
            {
                case 2:         
                    memset (GameKeyboard, false, sizeof(GameKeyboard));
                    return videoChange;
            }
        }

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}
	memset (GameKeyboard, false, sizeof(GameKeyboard));
    return videoChange;
}


// returns : 0
int DisplaySkillMenu (Player& player)
{
#define SKILLMENU_NUM 4

    int selectorMove  = 0;
    int selectorYDest = 0;
    int selItem       = GameInfo.gameLevelAdj;
    MenuItem menu[SKILLMENU_NUM] = 
    {
        {185, 210, "UBER GAMER"},
		{185, 250, "NORMAL GAMER"},
        {185, 290, "HOPELESS GAMER"},        
        {185, 330, "EASY ON ME, I'M YOUNG"}
    };

    PlayAreaRect item = {menu[selItem].x - 15, menu[selItem].y - 10, item.x1 + 355, item.y1 + 35,  item.x2 - item.x1, item.y2 - item.y1, 0, 0};
    BackgroundFlash selector;
    selector.palIdx = 64;

    // Setup Frame Border
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while (GameKeyboard[SDLK_ESCAPE] == false)
	{
		GamePlaySrnLock (player);	  
        // --------------------------

		stars_move ();
       
        for (int i = 0; i < SKILLMENU_NUM; i++)
            SDL_Printf (menu[i].x, menu[i].y, menu[i].text);

        selector.drawGameBorder (item, 10, 1);

        DisplayTitle (player, false);

        if (selectorMove == 0)
        {            
            if ((GameKeyboard[SDLK_DOWN] != false) && (selItem < SKILLMENU_NUM-1))
            {
                selectorMove  = 1;
                selectorYDest = menu[selItem+1].y-10;
                selItem++;
            }

            if ((GameKeyboard[SDLK_UP] != false) && (selItem > 0))
            {
                selectorMove  = -1;
                selectorYDest = menu[selItem-1].y-10;
                selItem--;
            }            
        }
        else if (selectorMove > 0)
        {
            if (item.y1 < selectorYDest)
            {
                item.y1 += 2;
                item.y2 += 2;
            }
            else
            {
                selectorMove = 0;
            }
        }
        else if (selectorMove < 0)
        {
            if (item.y1 > selectorYDest)
            {
                item.y1 -= 2;
                item.y2 -= 2;
            }
            else
            {
                selectorMove = 0;
            }
        }

        if (MenuActivate () != false)
        {
			GameInfo.gameLevelAdj = selItem;
			memset (GameKeyboard, false, sizeof(GameKeyboard));
			return selItem;
        }

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}

	memset (GameKeyboard, false, sizeof(GameKeyboard));
    return -1;
}


int DisplayCredits (Player& player)
{
char* InfoText[80] = 
{
GAMETITLE,
"WAS WRITTEN BY STEVEN DE TONI (2007)",
"",
"DEFEAT THE SPACE ALIENS OF THEIR",
"EVIL PLAN 9. BLAST THROUGH",
"30 WAVES OF ALIEN NASTIES.",
"",
"CAN YOU SURVIVE THE ONSLAUGHT!",
"",
"A GAME BASED ON :",
"LLAMATRON  -  JEFF MINTER",
"ROBOTRON  -  EUGENE JARVIS",
"",
"GRAPHICS AND SOUND FROM THE INTERNET.",
"MUSIC FROM THE COMMADORE 64 CLASSIC",
"MUTANTS, COMPOSER FRED GRAY.",
"\n"
};

    // Setup Frame Border
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

	int yStart = GamePlayArea.y2;
	int yEnd   = 100;

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while ((GameKeyboard[SDLK_ESCAPE] == false) && (MenuActivate () == false))
	{
		GamePlaySrnLock (player);	  
        // --------------------------

		stars_move ();

        DisplayTitle (player, false);

		int yPlot = yStart;
		int idx	  = 0;


 		while ((yPlot <= GamePlayArea.y2) && (InfoText[idx][0] != '\n'))
		{      
            int xPlot = GamePlayArea.xC - (SFont_TextWidth (PGameFontTitle, InfoText[idx]) / 2);
			SDL_Printf (PScreen, 
						   PGameFontTitle, 
                           xPlot, yPlot, 
						   InfoText[idx]);

            

			idx++;
			yPlot += SFont_TextHeight (PGameFontTitle) + 5;
		}
             

		if (yStart > yEnd)		
			yStart--;

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}

	memset (GameKeyboard, false, sizeof(GameKeyboard));
    return -1;
}



void DisplayEndScreen (Player& player)
{
    char  txt[][80] = 
    {
	    "CONGRADULATIONS",
        "YOU HAVE COMPLETED MAGNETRON",
	    "YOU ARE A TRUE UBER GAMER!",
        "",
        "PREPARE YOURSELF FOR PLAN 10!"
    };

    SDLSprite s[5];
    int showIdx = 0;
    int i;

    for (i = 0; i < 5; i++)
    {
        SDL_Surface* pBitmap = SDL_CreateRGBSurface (SDL_HWSURFACE, 
                                                     SFont_TextWidth  (PGameFontTitle, txt[i]),
                                                     SFont_TextHeight (PGameFontTitle), 
                                                     PScreen->format->BitsPerPixel, PScreen->format->Rmask, PScreen->format->Gmask, PScreen->format->Bmask, PScreen->format->Amask);                    

        SDL_Rect  r = {0, 0, pBitmap->w, pBitmap->h};
                
	    s[i].Create       (pBitmap, pBitmap->w, pBitmap->h, 1, SDL_TRUE);
	    s[i].SetColorKey  (0x00, 0x00, 0x00);
        SDL_FillRect   (pBitmap, &r, s[i].m_ColorKey);
        SDL_Printf     (pBitmap, PGameFontTitle, 0,0, txt[i]);    

        s[i].m_PosX = (float)(GamePlayArea.xC - (SFont_TextWidth  (PGameFontTitle, txt[i])/2) );
        s[i].m_PosY = (float)(GamePlayArea.yC - (((pBitmap->h+5)* 5)/2)) + ((SFont_TextHeight (PGameFontTitle) + 5) * i);

        ParticalTrig p = {PARTEVNT_SET_INT_DATA, NULL, &showIdx, i+1};
        PExpObjs->buildExplosionSplit (&s[i], PC_EXPOUT, true, 0.1f, GameFPS.rate + ((GameFPS.rate/2) * i), 1, &p);    
    }


	memset (GameKeyboard, false, sizeof(GameKeyboard));
    GamePlaying = false;
    bool playYeah = true;
    
	while ((GameKeyboard[SDLK_ESCAPE] == false) && (MenuActivate () == false))
	{
		GamePlaySrnLock (player);	  
		// --------------------------

        stars_move ();

        for (int i = 0; i < showIdx; i++)
            s[i].Draw (PScreen);

        if ((showIdx == 3) && (playYeah != false))
        {
            playYeah = false;
            Mix_PlayChannel(SNDCHL_PLAYER, GameSNDResource[GRS_OH_YEAH].pResSnd, 0);
        }

        PExpObjs->moveShowAll (PScreen);
	
		// --------------------------
		GamePlaySrnUnlock (player); 	
	}

    for (i = 0; i< 5; i++)
        PExpObjs->buildExplosionSplit (&s[i], PC_EXPOUT, false, 0.5f, (GameFPS.rate*3));

	Mix_PlayChannel(SNDCHL_PLAYER, GameSNDResource[GRS_SMARTBOMB1].pResSnd, 0);
    
    while (PExpObjs->listNo > 0)
	{
		GamePlaySrnLock (player);	        
		// --------------------------
		       
        stars_move ();

		PExpObjs->moveShowAll (PScreen);
			
		// --------------------------
		GamePlaySrnUnlock (player);
	}

    GamePlaying = true;
	memset (GameKeyboard, false, sizeof(GameKeyboard));
}


void DisplayHighScoreScreen (Player& player, bool enterName)
{
    char* pLines[80] = 
    {
        "ABCDEFGHIJKLMN",
        "OPQRSTUVWXYZ -",
        "SAVE"
    };
	
	int replIdx = -1;
	int idx;	

	if (enterName != false)
	{
		// Determine score position...
		replIdx = 0;
		for (idx = 9; idx >= 0; idx--)
		{	
			if (player.playersScore > GameInfo.scores[idx].score)
				continue;
			else if ((idx < 9) && (idx >= 0))
			{
				replIdx = idx+1;
				break;
			}
			else 
			{
				replIdx = idx;
				break;
			}
		}

		// not a worthy player!
		if ((replIdx == 9) && (player.playersScore < GameInfo.scores[9].score))
			return;

		// shuffle down score
		for (idx = 9; idx > replIdx; idx--)
		{
			GameInfo.scores[idx] = GameInfo.scores[idx-1];
		}

		GameInfo.scores[replIdx].name[0] = '\0';
		GameInfo.scores[replIdx].score   = player.playersScore;
		GameInfo.scores[replIdx].wave    = player.gameLevel+1;
	}

	memset (GameKeyboard, false, sizeof(GameKeyboard));
    GamePlaying = false;
    
    // Setup Frame Border
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

    int selectorYMove = 0;
    int selectorYDest = 0;
    int selectorXMove = 0;
    int selectorXDest = 0;
    int selItem       = 0;
    int selLine       = 0;
    int selMaxWidth   = 15;
    int selMaxHeight  = 2;
	int blinkCursor   = 0;

    int txtXPos       = 35;
    int txtYPos       = 375;
    int txtWidth      = 42;
    int txtHeight     = 35;
	int frameCount    = (GameFPS.rate * 15);
	int yPlotPos	  = 0;
	int yScrollDir    = -5;
 
	if (enterName == false)
		yPlotPos = PScreen->h;	

    PlayAreaRect item = {txtXPos-5,              txtYPos-5, 
                         item.x1 + 36,           item.y1 + 26,                           
                         item.x2 - item.x1, item.y2 - item.y1, 0, 0};

    PlayAreaRect saver = {txtXPos-5,              txtYPos-5, 
                          item.x1 + 36,           item.y1 + 26,                           
                          item.x2 - item.x1, item.y2 - item.y1, 0, 0};

    BackgroundFlash selector;

	while ((GameKeyboard[SDLK_ESCAPE] == false))
	{
		GamePlaySrnLock (player);		
		// --------------------------

		if (enterName == false)
		{
			// Scroll up/down if need be...
			yPlotPos += yScrollDir;
			if (yPlotPos < 100)
				yPlotPos = 100;
		}
			
		frameCount--;

		if ((enterName == false) && (frameCount < 0))
		{
			if (yScrollDir > 0)
				break;

			yScrollDir = abs(yScrollDir);
			frameCount = GameFPS.rate * 2;
		}

		stars_move ();

		DisplayTitle (player, false);

	    PExpObjs->showAll (PScreen);       

	
		if (enterName == false)
		{
			int w = SFont_TextWidth  (PGameFontTitle, "HIGH SCORE TABLE"); 
			SDL_Printf (PScreen, PGameFontTitle, 
						GamePlayArea.xC - (w/2), yPlotPos+40, 
						"%s", 
						"HIGH SCORE TABLE");
		}

		// Display Current High scores...
		for (int idx = 0; idx < 10; idx++)
		{
			SDL_Printf (PScreen, PGameFontTitle, 
						40, yPlotPos + 100 + ((SFont_TextHeight (PGameFontTitle) + 10) * idx), 
						"%02d",  GameInfo.scores[idx].wave);

			SDL_Printf (PScreen, PGameFontTitle, 
						90, yPlotPos + 100 + ((SFont_TextHeight (PGameFontTitle) + 10) * idx), 
						"%d", GameInfo.scores[idx].score);

			SDL_Printf (PScreen, PGameFontTitle, 
						260, yPlotPos + 100 + ((SFont_TextHeight (PGameFontTitle) + 10) * idx), 
						"%s", GameInfo.scores[idx].name);

			if ((idx == replIdx) && (enterName != false))
			{
				// Display cursor
				int w = SFont_TextWidth  (PGameFontTitle, GameInfo.scores[replIdx].name);
				if (blinkCursor == 0)
				{
					blinkCursor = GameFPS.rate / 4;
				}
				else if (blinkCursor == 1)
				{
					blinkCursor = -(int)(GameFPS.rate / 4);
				}
				else if (blinkCursor > 0)
				{
					SDL_Printf (PScreen, PGameFontTitle, 
								260 + w, 
								yPlotPos + 100 + ((SFont_TextHeight (PGameFontTitle) + 10) * idx), "-");
					blinkCursor--;
				}
				else
					blinkCursor++;
			}
		}

		if (enterName != false)
		{
            int i;
			for (i = 0; i < 2; i++)
			{
				for (int idx = 0; pLines[i][idx] != '\0'; idx++)
				{
					SDL_Printf (PScreen, PGameFontTitle, 
										   txtXPos + (txtWidth  * idx), 
								yPlotPos + txtYPos + (txtHeight * i), 
								"%c",
								pLines[i][idx]);
				}
			}


            int xPlot = GamePlayArea.xC - (SFont_TextWidth (PGameFontTitle, pLines[i]) / 2);

            SDL_Printf (PScreen, PGameFontTitle, 
					    xPlot, yPlotPos + txtYPos + (txtHeight * i), 
			            "%s",
			            pLines[i]);

            if (selLine < 2)
			    selector.drawGameBorder (item, 4, 1);
            else
                selector.drawGameBorder (saver, 4, 1);

			// LEFT/ RIGHT
			if (selectorXMove == 0)
			{            
                if (selLine < 2)
                {
				    if ((GameKeyboard[SDLK_RIGHT] != false) && (selItem < 13))
				    {
					    selItem++;
					    selectorXMove = 1;
					    selectorXDest =  item.x1 + txtWidth;
				    }

				    if ((GameKeyboard[SDLK_LEFT] != false) && (selItem > 0))
				    {
					    selItem--;
					    selectorXMove = -1;
					    selectorXDest =  item.x1 - txtWidth;
				    }    
                }
            
			}
			else if (selectorXMove > 0)
			{
				if (item.x1 < selectorXDest)
				{
					item.x1 += 3;
					item.x2 += 3;
				}
				else
					selectorXMove = 0;
			}
			else if (selectorXMove < 0)
			{
				if (item.x1 > selectorXDest)
				{
					item.x1 -= 3;
					item.x2 -= 3;
				}
				else
					selectorXMove = 0;
			}

			// UP / DOWN
			if (selectorYMove == 0)
			{         
				if ((GameKeyboard[SDLK_DOWN] != false) && (selLine == 1))
				{
					selLine++;

                    saver.x1 = xPlot - 10;
                    saver.x2 = saver.x1 + SFont_TextWidth (PGameFontTitle, pLines[i]) + 20;                    
                    saver.y1 = yPlotPos + txtYPos + (txtHeight * i) - 10;
                    saver.y2 = yPlotPos + txtYPos + (txtHeight * i+1) + 25;
                    saver.w = saver.x2 - saver.x1;
                    saver.h = saver.y2 - saver.y1;
                    saver.xC = saver.yC = 0;
                    memset (GameKeyboard, false, sizeof(GameKeyboard));
				}

				if ((GameKeyboard[SDLK_UP] != false) && (selLine == 2))
				{
					selLine--;
                    memset (GameKeyboard, false, sizeof(GameKeyboard));
				}

                
				if ((GameKeyboard[SDLK_DOWN] != false) && (selLine < 1))
				{
					selLine++;
					selectorYMove = 1;
					selectorYDest = item.y1 + txtHeight;
				}

				if ((GameKeyboard[SDLK_UP] != false) && (selLine > 0))
				{
					selLine--;
					selectorYMove = -1;
					selectorYDest = item.y1 - txtHeight;
				}                     
			}
			else if (selectorYMove > 0)
			{
				if (item.y1 < selectorYDest)
				{
					item.y1 += 3;
					item.y2 += 3;
				}
				else
					selectorYMove = 0;
			}
			else if (selectorYMove < 0)
			{
				if (item.y1 > selectorYDest)
				{
					item.y1 -= 3;
					item.y2 -= 3;
				}
				else
					selectorYMove = 0;
			}

			if (MenuActivate() != false) 
			{
                if (selLine < 2)
                {
				    int len = strlen(GameInfo.scores[replIdx].name);

				    if (pLines[selLine][selItem] == '-')
				    {				
					    if (len > 0)
					    {
						    GameInfo.scores[replIdx].name[len-1] = '\0';
					    }
				    }
				    else if (len < sizeof(GameInfo.scores[replIdx].name)-1)
				    {
					    char c[2] = {pLines[selLine][selItem], '\0'};
					    strcat (GameInfo.scores[replIdx].name, c);
				    }
                }
                else
                    break;

				memset (GameKeyboard, false, sizeof(GameKeyboard));
			}
		}
		else if (MenuActivate () != false)
			break;

		// --------------------------
		GamePlaySrnUnlock (player, false); 		
	}   

	if (enterName != false)
	{
		if (GameInfo.scores[replIdx].name[0] == '\0')
			strcpy (GameInfo.scores[replIdx].name, GAMETITLE);
	}

	memset (GameKeyboard, false, sizeof(GameKeyboard));
}



#ifndef _XBOX
// returns :
// 0 = play game
// 1 = quit game
// 2 = change window/screen options.
int DisplayMainMenu (Player& player)
{
#define MAINMENU_NUM 8
    int selectorMove  = 0;
    int selectorYDest = 0;
    int selItem       = 0;
	int frameCount    = (GameFPS.rate * 15);
	int yPlotPos	  = PScreen->h;
	int yScrollDir    = -5;

    MenuItem menu[MAINMENU_NUM] = 
    {
        {185, 130, "PLAY GAME"},
		{185, 170, "SKILL LEVEL"},
        {185, 210, "CONFIG KEYBOARD"},
        {185, 250, "CONFIG JOYSTICK"},
        {185, 290, "CONFIG VIDEO"},
		{185, 330, "CREDITS"},
		{185, 370, "HIGH SCORES"},
        {185, 410, "QUIT GAME"},
    };

    PlayAreaRect item = {menu[selItem].x - 15, menu[selItem].y - 10, item.x1 + 285, item.y1 + 35,  item.x2 - item.x1, item.y2 - item.y1, 0, 0};
    BackgroundFlash selector;


    // Load background music
    PBGMusic = Mix_LoadMUS(_SPATH "sounds" _SEP "mmmusic.ogg");

    // Load and Prepare the font
	SDL_Surface* f = IMG_Load (_SPATH "fonts" _SEP "sfont_title.png");

    if (f == NULL)
	    return -100; 

	if ((PFontMenuTitle = SFont_InitFont(SDL_DisplayFormat(f))) == NULL)
        return -101;

    SDL_FreeSurface(f);    

    // Setup Frame Border
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

    selector.palIdx = 64;

    DisplayTitle (player, true);

	DisplayHighScoreScreen (player, true);

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while (GameKeyboard[SDLK_ESCAPE] == false)
	{
		GamePlaySrnLock (player);	  
        // --------------------------

		// Scroll up/down if need be...
		yPlotPos += yScrollDir;
		if (yPlotPos < 0)
			yPlotPos = 0;
			
		frameCount--;

		if (frameCount < 0)
		{
			if (yScrollDir > 0)
			{
				DisplayHighScoreScreen (player, false);
				yScrollDir = -yScrollDir;
				frameCount = (GameFPS.rate * 15);
			}
			else
			{
				yScrollDir = abs(yScrollDir);
				frameCount = GameFPS.rate * 2;
			}
		}

		stars_move ();
        
        for (int i = 0; i < MAINMENU_NUM; i++)
            SDL_Printf (menu[i].x, yPlotPos+menu[i].y, menu[i].text);

		if (yPlotPos == 0)
			selector.drawGameBorder (item, 10, 1);

        DisplayTitle (player, false);

        if ((selectorMove == 0) && (yPlotPos == 0)) 
        {            
            if ((GameKeyboard[SDLK_DOWN] != false) && (selItem < MAINMENU_NUM-1))
            {
                selectorMove  = 1;
                selectorYDest = menu[selItem+1].y-10;
                selItem++;
				frameCount = (GameFPS.rate * 15);
            }

            if ((GameKeyboard[SDLK_UP] != false) && (selItem > 0))
            {
                selectorMove  = -1;
                selectorYDest = menu[selItem-1].y-10;
                selItem--;
				frameCount = (GameFPS.rate * 15);
            }     			
        }
        else if (selectorMove > 0)
        {
            if (item.y1 < selectorYDest)
            {
                item.y1 += 2;
                item.y2 += 2;
            }
            else
            {
                selectorMove = 0;
            }
        }
        else if (selectorMove < 0)
        {
            if (item.y1 > selectorYDest)
            {
                item.y1 -= 2;
                item.y2 -= 2;
            }
            else
            {
                selectorMove = 0;
            }
        }

        if (MenuActivate () != false)
        {
            switch (selItem)
            {
                case 0:
                    SFont_FreeFont(PFontMenuTitle);
                    Mix_HaltMusic();
                    Mix_FreeMusic(PBGMusic);
                    return 0;

				case 1:
					DisplaySkillMenu (player);
					break;
                case 2:
                    DisplayKeyboardMenu (player);
                    memset (GameKeyboard, false, sizeof(GameKeyboard));
                    break;

				case 3:
					DisplayJoystickMenu (player);					
					break;

				case 5:
					DisplayCredits (player);
					break;

				case 6:
					DisplayHighScoreScreen (player, false);
					break;

                case 7:
                    SFont_FreeFont(PFontMenuTitle);
                    Mix_HaltMusic();
                    Mix_FreeMusic(PBGMusic);
                    return 1;

                case 4:
                    if (DisplayVideoMenu (player) != false)
                    {
                        SFont_FreeFont(PFontMenuTitle);
                        Mix_HaltMusic();
                        Mix_FreeMusic(PBGMusic);
	                    memset (GameKeyboard, false, sizeof(GameKeyboard));
                        return 2;
                    }
                    break;
            }
        }

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}
    SFont_FreeFont(PFontMenuTitle);
    Mix_HaltMusic();
    Mix_FreeMusic(PBGMusic);
	memset (GameKeyboard, false, sizeof(GameKeyboard));
    return 1;
}
#endif

void ReadConfigFile (void)
{
    FILE* f = fopen (_SPATH "magnetron.cfg", "rb");

    if (f == NULL)
        return;

    fread (&GameInfo, sizeof (GameInfo), 1, f);
    fclose (f);

#ifdef _XBOX
    XBOX_ReadConfigScreenPos();
#endif    
}

void WriteConfigFile (void)
{
    FILE* f = fopen (_SPATH "magnetron.cfg", "wb");

    if (f == NULL)
        return;

    fwrite (&GameInfo, sizeof (GameInfo), 1, f);

    fclose (f);

#ifdef _XBOX
    XBOX_WriteConfigScreenPos ();
#endif    
}

// ------------------------ XBOX Screen Position Funcs -------------------

#ifndef _XBOX
DECLSPEC void SDLCALL SDL_XBOX_SetScreenPosition(float x, float y)   {}
DECLSPEC void SDLCALL SDL_XBOX_SetScreenStretch(float xs, float ys)  {}
DECLSPEC void SDLCALL SDL_XBOX_GetScreenPosition(float *x, float *y) {}
DECLSPEC void SDLCALL SDL_XBOX_GetScreenStretch(float *xs, float *ys){}
#endif

float XScreenPos    = 0;
float YScreenPos    = 0;
float XScreenStetch = 0;
float YScreenStetch = 0;

#ifdef _XBOX
// returns :
// 0 = play game
// 1 = quit game
// 2 = change window/screen options.
int DisplayMainMenu (Player& player)
{
#define MAINMENU_NUM 6
    int selectorMove  = 0;
    int selectorYDest = 0;
    int selItem       = 0;
	int frameCount    = (GameFPS.rate * 15);
	int yPlotPos	  = PScreen->h;
	int yScrollDir    = -5;

    MenuItem menu[MAINMENU_NUM] = 
    {
        {185, 130, "PLAY GAME"},
		{185, 170, "SKILL LEVEL"},
		{185, 210, "CONFIG T.V VIEW"},
		{185, 250, "CREDITS"},
		{185, 290, "HIGH SCORES"},
        {185, 330, "QUIT GAME"},
    };

    PlayAreaRect item = {menu[selItem].x - 15, menu[selItem].y - 10, item.x1 + 285, item.y1 + 35,  item.x2 - item.x1, item.y2 - item.y1, 0, 0};
    BackgroundFlash selector;


    // Load background music
    PBGMusic = Mix_LoadMUS(_SPATH "sounds" _SEP "mmmusic.ogg");

    // Load and Prepare the font
	SDL_Surface* f = IMG_Load (_SPATH "fonts" _SEP "sfont_title.png");

    if (f == NULL)
	    return -100; 

	if ((PFontMenuTitle = SFont_InitFont(SDL_DisplayFormat(f))) == NULL)
        return -101;

    SDL_FreeSurface(f);    

    // Setup Frame Border
    GamePlayArea.x1 = 5;
    GamePlayArea.y1 = 5;
    GamePlayArea.x2 = PScreen->w - GamePlayArea.x1;
    GamePlayArea.y2 = PScreen->h - 5;
    GamePlayArea.w  = GamePlayArea.x2 - GamePlayArea.x1;
    GamePlayArea.h  = GamePlayArea.y2 - GamePlayArea.y1;
    GamePlayArea.xC = (PScreen->w/2) - GamePlayArea.x1;
    GamePlayArea.yC = (PScreen->h/2) - GamePlayArea.y1;

    selector.palIdx = 64;

    DisplayTitle (player, true);

	DisplayHighScoreScreen (player, true);

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while (GameKeyboard[SDLK_ESCAPE] == false)
	{
		GamePlaySrnLock (player);	  
        // --------------------------

		// Scroll up/down if need be...
		yPlotPos += yScrollDir;
		if (yPlotPos < 0)
			yPlotPos = 0;
			
		frameCount--;

		if (frameCount < 0)
		{
			if (yScrollDir > 0)
			{
				DisplayHighScoreScreen (player, false);
				yScrollDir = -yScrollDir;
				frameCount = (GameFPS.rate * 15);
			}
			else
			{
				yScrollDir = abs(yScrollDir);
				frameCount = GameFPS.rate * 2;
			}
		}

		stars_move ();
        
        for (int i = 0; i < MAINMENU_NUM; i++)
            SDL_Printf (menu[i].x, yPlotPos+menu[i].y, menu[i].text);

		if (yPlotPos == 0)
			selector.drawGameBorder (item, 10, 1);

        DisplayTitle (player, false);

        if ((selectorMove == 0) && (yPlotPos == 0)) 
        {            
            if ((GameKeyboard[SDLK_DOWN] != false) && (selItem < MAINMENU_NUM-1))
            {
                selectorMove  = 1;
                selectorYDest = menu[selItem+1].y-10;
                selItem++;
				frameCount = (GameFPS.rate * 15);
            }

            if ((GameKeyboard[SDLK_UP] != false) && (selItem > 0))
            {
                selectorMove  = -1;
                selectorYDest = menu[selItem-1].y-10;
                selItem--;
				frameCount = (GameFPS.rate * 15);
            }     			
        }
        else if (selectorMove > 0)
        {
            if (item.y1 < selectorYDest)
            {
                item.y1 += 2;
                item.y2 += 2;
            }
            else
            {
                selectorMove = 0;
            }
        }
        else if (selectorMove < 0)
        {
            if (item.y1 > selectorYDest)
            {
                item.y1 -= 2;
                item.y2 -= 2;
            }
            else
            {
                selectorMove = 0;
            }
        }

        if (MenuActivate () != false)
        {
            switch (selItem)
            {
                case 0:
                    SFont_FreeFont(PFontMenuTitle);
                    Mix_HaltMusic();
                    Mix_FreeMusic(PBGMusic);
                    return 0;

				case 1:
					DisplaySkillMenu (player);
					break;

                case 2:
					XBOX_SetConfigScreenPos (player);
                    break;


				case 3:
					DisplayCredits (player);
					break;

				case 4:
					DisplayHighScoreScreen (player, false);
					break;

                case 5:
                    SFont_FreeFont(PFontMenuTitle);
                    Mix_HaltMusic();
                    Mix_FreeMusic(PBGMusic);
                    return 1;
            }
        }

		// --------------------------
		GamePlaySrnUnlock (player, false); 
		
	}
    SFont_FreeFont(PFontMenuTitle);
    Mix_HaltMusic();
    Mix_FreeMusic(PBGMusic);
	memset (GameKeyboard, false, sizeof(GameKeyboard));
    return 1;
}
#endif

void XBOX_ReadConfigScreenPos  (void)
{
    SDL_XBOX_GetScreenPosition (&XScreenPos,    &YScreenPos);
    SDL_XBOX_GetScreenStretch  (&XScreenStetch, &YScreenStetch);

    //Open the config file.
    FILE* f = fopen(_SPATH "screenpos.cfg", "r");
    if (f != NULL)
    {
        fscanf (f, "%f %f %f %f", &XScreenPos, &YScreenPos, &XScreenStetch, &YScreenStetch);
        fclose (f);
    }
}

void XBOX_WriteConfigScreenPos (void)
{
    SDL_XBOX_GetScreenPosition (&XScreenPos,    &YScreenPos);
    SDL_XBOX_GetScreenStretch  (&XScreenStetch, &YScreenStetch);

    //Open the config file.
    FILE* f = fopen(_SPATH "screenpos.cfg", "w");
    if (f != NULL)
    {
        fprintf (f, "%f %f %f %f", XScreenPos, YScreenPos, XScreenStetch, YScreenStetch);
        fclose (f);
    }
}

void XBOX_SetScreenPos (void)
{
    //now do the stuff
    //D3D_Device->SetFlickerFilter(flickerlevel);
    //now resize our screen
    SDL_XBOX_SetScreenPosition (XScreenPos,    YScreenPos);
    SDL_XBOX_SetScreenStretch  (XScreenStetch, YScreenStetch);
}

void XBOX_SetConfigScreenPos   (Player& player)
{
    int  mode = 0;
	bool changed = false;

    SDL_XBOX_GetScreenPosition (&XScreenPos,    &YScreenPos);
    SDL_XBOX_GetScreenStretch  (&XScreenStetch, &YScreenStetch);

	memset (GameKeyboard, false, sizeof(GameKeyboard));
	while ((GameKeyboard[SDLK_ESCAPE] == false) && (mode < 2))
	{
		if (changed != false)
		{
            SDL_XBOX_SetScreenPosition (XScreenPos,    YScreenPos);
            SDL_XBOX_SetScreenStretch  (XScreenStetch, YScreenStetch);
			changed = false;
		}
		GamePlaySrnLock (player);	

		stars_move ();
	
		if (GameKeyboard[GameInfo.userKeys[USRKEY_LEFT]] != false)
        {
            if (mode == 0)            
                XScreenPos--;
            else
                XScreenStetch--;
			changed = true;
        }
        else if (GameKeyboard[GameInfo.userKeys[USRKEY_RIGHT]] != false)
        {
            if (mode == 0)            
                XScreenPos++;
            else
                XScreenStetch++;
			changed = true;
        }                    
		if (GameKeyboard[GameInfo.userKeys[USRKEY_UP]] != false)
        {
            if (mode == 0)            
                YScreenPos--;
            else
                YScreenStetch--;
			changed = true;
        }
        else if (GameKeyboard[GameInfo.userKeys[USRKEY_DOWN]] != false)
        {
            if (mode == 0)            
                YScreenPos++;
            else
                YScreenStetch++;		
			changed = true;
        }     
		else if (MenuActivate () != false)
		{
			memset (GameKeyboard, false, sizeof(GameKeyboard));
            mode++;
		}

        switch (mode)
        {
            case 0:
                SDL_Printf (30, 50,   "USE JOYPAD TO ADJUST TOP-LEFT");
                SDL_Printf (140, 80,  "SCREEN POSITION");
                SDL_Printf (120, 140, "(PRESS FIRE TO SET)");               
                SDL_Printf (140, 200, "TOP X:%d", (int)XScreenPos);
                SDL_Printf (140, 230, "TOP Y:%d", (int)YScreenPos);
                break;

            case 1:
                SDL_Printf (50,  330, "USE JOYPAD TO ADJUST BOTTOM-RIGHT");
                SDL_Printf (180, 360, "SCREEN POSITION");
                SDL_Printf (160, 420, "(PRESS FIRE TO SET)");               
                SDL_Printf (140, 200, "BOT X:%d", (int)XScreenStetch);
                SDL_Printf (140, 230, "BOT Y:%d", (int)YScreenStetch);
                break;
        }

        
		// --------------------------

		GamePlaySrnUnlock (player, false); 		
	}

    memset (GameKeyboard, false, sizeof(GameKeyboard));    
}
