#ifndef  _OBJS_
#define  _OBJS_

#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include <SDL/SDL_image.h>
#include "lib/SDLSprite.h"
#include "lib/SDLSpriteList.h"
#include "lib/SDL_framerate.h"
#include "lib/SFont.h"
#include "specpal.h"

#ifdef _XBOX
#	define _SPATH "d:\\"
#	define _SEP "\\"
#	define PARTBLKSIZE 2
#elif _WIN32
#	define _SEP "\\"
#	define _SPATH ""
#	define PARTBLKSIZE 1
#else
#	define _SEP "/"
#	define _SPATH ""
#	define PARTBLKSIZE 1
#endif

#pragma warning(disable:4096) // stop complainly about C4096: '__cdecl' must be used with '...'

#define max(a,b) (((a) > (b)) ? (a) : (b))
#define min(a,b) (((a) < (b)) ? (a) : (b))
#define abs(a) (((a)<0) ? -(a) : (a))
#define sign(a) (((a)<0) ? -1 : (a)>0 ? 1 : 0)
#define MAX_INT 2147483647

class Partical;
class ParticalCollection;
class Player;
class BackgroundFlash;

enum  ObjectType {OBJT_NULL       = 0,
                  OBJT_BET        = 100,  BET_BLOB1, BET_BLOB2, BET_BLOB3, BET_BLOB_SPLITTER, BET_BLOB_SHIP1, BET_BLOB_PODS, BET_BLOB_PODLETS, BET_SHOT1, BET_ELECTRODE1, BET_BOSS_SHEILD_ROTATE, BET_SKULLCROSS1, BET_SHOT_REFIRE, BET_HEDGEHOG, BET_HEDGEHOG_SHOTL, BET_HEDGEHOG_SHOTU, BET_HEDGEHOG_SHOTR, BET_SHIPSHOT1,
				  OBJT_AET        = 200,  AET_MUTANT1, AET_BRAIN1, AET_BRAIN2, AET_SPAWNER1, AET_ROBOTBULLY1, AET_BOSS1, AET_LASER_HORZ, AET_LASER_VERT, AET_BULL_DROP, AET_BOSS_BRAIN, AET_SHIPSHOOTER, AET_ORB_BIG, AET_IMPOSROBOT1, AET_ORB_SMALL, AET_BIG_BRIAN_BOSS, AET_ORB_OF_POW,
				  OBJT_DET        = 300,  DET_HOMERSHOT1, DET_HOMERSHOT2, DET_ROBBYZOMBIE1, DET_BONUS, DET_ROBOTELEC1,
				  OBJT_WKO        = 400,  WKO_MAX, WKO_SAM, WKO_WENCH, WKO_POWEXTRALIFE, WKO_POW3FIRE, WKO_POWFASTERFIRE, WKO_POWSMARTBOMB, WKO_POWATTRACT, WKO_POWSHOCKBLASTER, WKO_ROBBYZOMBIE, WKO_RAIN1,
                  OBJT_PLY        = 1000, PLY_BULLET_LL,  PLY_BULLET_LU,  PLY_BULLET_UU, PLY_BULLET_RU, PLY_BULLET_RR, PLY_BULLET_RD, PLY_BULLET_DD, PLY_BULLET_LD
};

typedef struct
{
    char name[15];
    int  score;
    int  wave;
} GameHighScore;

typedef struct
{
    bool   fullScreen;
    int    bitDepth; // 16/32
	int    gameLevelAdj;
    SDLKey userKeys[8];
    int	   userJoy[8];
    
    GameHighScore scores[10];
} GameConfig;

typedef struct
{
	SDLSprite* pResObj;

    char	   filename[256];
    int		   frameWidth;
    int	       frameHeight;
    int        frameNum;
    int        ckr, ckg, ckb;
	int        hitPoints;
	int        score;
} BitmapResources;

typedef struct
{
	Mix_Chunk* pResSnd;
	char	   filename[256];
} SoundResources;

// ****** Sound Resource Types ******
enum SoundType	   {SNDT_NULL = 0,
					SNDT_SHOOT, 
					SNDT_HIT,
					SNDT_DEATH,
                    SNDT_INTRO,
					SNDT_MOVE};

enum SoundChannels {SNDCHL_PLAYER, 
                    SNDCHL_GAMEMISC,
					SNDCHL_ENEMY_SHOOT1, 
                    SNDCHL_ENEMY_SHOOT2, 
					SNDCHL_ENEMY_HIT,
					SNDCHL_ENEMY_DEATH,
                    SNDCHL_PICKFRIEND,
                    SNDCHL_EXTRALIFE,
					SNDCHL_SMARTBOMB,
                    SNDCHL_POW,
					SNDCHL_POWSHOCKBLAST,
                    SNDCHL_MUTANTSPAWN,
                    SNDCHL_MAIMEHIT
					};					
						

#define GRS_PLAYER_SHOOT       0
#define GRS_PLAYER_DEATH       1
#define GRS_OH_YEAH            2
#define GRS_PICKUP1            3
#define GRS_PICKUP2            4
#define GRS_PICKUP3            5
#define GRS_ENEMYHIT1          6
#define GRS_ENEMYDEATH1        7
#define GRS_SMARTBOMB1         8
#define GRS_ELECTRIC1          9
#define GRS_OOW                10
#define GRS_PLAYERUP           11
#define GRS_ENDLEVEL           12
#define GRS_MUTANTSPAWN        13
#define GRS_ELECTRICDEATH      14
#define GRS_MENUINTRO1         15
#define GRS_MENUINTRO2         16
#define GRS_LASER1             17
#define GRS_MAIMEHIT           18 
#define GRS_EXPLODE1		   19
#define GRS_ENEMYSHOOT1		   20
#define GRS_EXPLODE2		   21

// **********************************

#include "player.h"

// -========== External Globals ==============-
extern SDL_Surface*        PScreen;
extern SFont_Font*	       PGameFontTitle;
extern SFont_Font*	       PGameFontInfoBonus[];
extern SDL_Rect		       GameScreenRect;
extern FPSmanager	       GameFPS;
extern BitmapResources     GameBMResource[];
extern SoundResources      GameSNDResource[];
extern SDLKey			   GameKeyboard[512];
extern BackgroundFlash	   GameFlasher;
extern GameConfig          GameInfo;
extern bool                GamePlaying;

extern ParticalCollection* PExpObjs;
extern SDLSpriteList*	   PEnemyList;
extern SDLSpriteList*      PFriendsList;
extern SDLSpriteList*	   PElectrodesList;
extern SDLSpriteList*	   PPlayerBulletList;

// *** sin/cos table in radians ***
#define SINCOS_TABNUM   1024
#define PI		        3.1415926535897932384626433832795f
#define SINCOS_RADSEGMT ((PI*2)/SINCOS_TABNUM)
#define ATAN2(y,x)	    (atan2(y,x)/SINCOS_RADSEGMT)
extern float SIN[SINCOS_TABNUM];
extern float COS[SINCOS_TABNUM];
// -==========================================-


// ***** Bitmap Resource Types *****
#define GR_PLAYER           0
#define GR_PLAYER_BULLET0   1
#define GR_PLAYER_BULLET1   2
#define GR_PLAYER_BULLET2   3
#define GR_PLAYER_BULLET3   4

#define GR_BASIC_ENEMY1     5
#define GR_ADV_MUTANT1      6

#define GR_BASIC_ENEMYSHOT1 7
#define GR_DYN_HOMERSHOT1   8
#define GR_ADV_BRAIN1       9

#define GR_MAX              10
#define GR_SAM              11
#define GR_ROBBY            12
#define GR_BE_ELECTRODE1    13

#define GR_POW_EXTRALIFE    14
#define GR_POW_3FIRE        15
#define GR_POW_FASTERFIRE   16
#define GR_POW_SMARTBOMB    17
#define GR_POW_SHOCKBLASTER 18
#define GR_POW_ATTRACTOR    19
#define GR_POW_ATTRACTMAG   20
#define GR_BASIC_ENEMY2     21
#define GR_BASIC_ENEMY3     22
#define GR_ADV_SPAWNER      23
#define GR_ADV_ROBOTBULLY1  24
#define GR_BASIC_SKULLCROSS 25
#define GR_BASIC_ENENYSHOT2 26
#define GR_BASIC_SPLITTER   27
#define GR_BASIC_ENEMY5     28
#define GR_BASIC_IMPOSROBOT1   29
#define GR_BASIC_ENEMY_PODS    30
#define GR_BASIC_ENEMY_PODLETS 31
#define GR_ADV_BOSS1        32
#define GR_WENCH            33
#define GR_FRD_RAIN1        34
#define GR_ADV_LASER_HORZ   35
#define GR_ADV_LASER_VERT	36
#define GR_ADV_BULLDROP     37
#define GR_BASIC_HEADGEHOG  38
#define GR_BASIC_SHIPSHOT   39
#define GR_ADV_ORB          40
#define GR_ADV_ORB2		    41
#define GR_ADV_ORB_SMALL    42
#define GR_ADV_BIG_BRIAN    43

// **********************************

// **** User Key and Joy Stick Array Mappings ****
#define USRKEY_LEFT      0
#define USRKEY_UP        1
#define USRKEY_RIGHT     2
#define USRKEY_DOWN      3
#define USRKEY_FIRELEFT  4
#define USRKEY_FIREUP    5
#define USRKEY_FIRERIGHT 6
#define USRKEY_FIREDOWN  7

#define USRJOY_LEFT      -1
#define USRJOY_UP        -2
#define USRJOY_RIGHT     -3
#define USRJOY_DOWN      -4
#define USRJOY_BUT1	     0  
#define USRJOY_BUT2	     1
#define USRJOY_BUT3	     2
#define USRJOY_BUT4	     3
#define USRJOY_BUT5	     4
#define USRJOY_BUT6	     5
#define USRJOY_BUT7	     6
#define USRJOY_BUT8	     7
#define USRJOY_BUT9	     8
#define USRJOY_BUT10     9
// **********************************

// -==== Global function prototypes  ====- 
// ------------------------
void SDL_ColPrintf (SDL_Surface* pS, SFont_Font* pF, Uint8 r, Uint8 g, Uint8 b, int colidx, int x, int y, char* pszFormat, ...);
void SDL_Printf(int x, int y, char* pszFormat, ...);
void SDL_Printf(SDL_Surface* pS, SFont_Font* pF, int x, int y, char* pszFormat, ...);
void DbgPrintf (char* pszFormat, ...);
void GamePlaySrnUnlock (Player& player, bool dispPlayerScore = true); // screen unlock
void GamePlaySrnLock   (Player& player, bool clearScreen = true);// screen unlock
// ------------------------

void BuildCosSinTable            (void);
void SDL_line                    (SDL_Surface *s, int x1, int y1, int x2, int y2, Uint32 color);
void SDL_Lightning			     (SDL_Surface* s, int bx1, int by1, int bx2, int by2, int steps, int range);
void ProcessParticalTrigEvent    (Partical* pObj);
void KillFriendlySkullCrossbones (SDLSprite* pNode);
void CheckSpawnCreateOnDeath     (SDLSprite* pNode, ObjectType bulletHitType = OBJT_NULL);
void DrawPixel					 (SDL_Surface* pScreen, int x, int y, Uint8 R, Uint8 G, Uint8 B);

void SNDPlayPan               (int channel, Mix_Chunk* chunk, int loops, int xPos, int yPos);

// Fast integer square root 
void SQRTIntInit (void);
unsigned int SQRTInt (unsigned int a);
// -=====================================-


// -===================================================================-

enum ParticalType     {PART_BASIC    = 0};
enum PartialEventType {PARTEVNT_NULL = 0, 
					   PARTEVNT_ADD2SPRITELIST, 
					   PARTEVNT_ADD2SPRITEARRAY,
                       PARTEVNT_SET_BOOL_DATA,
                       PARTEVNT_SET_INT_DATA};

typedef struct
{
    PartialEventType   type;
    void*              pListData;
	void*              pData;
	int				   data;	
} ParticalTrig;

class Partical
{
    public:
        SDL_Surface* pBitmap;

        float         xSpeed, ySpeed, x, y;
        int           life;
        ParticalType  type;
		ParticalTrig* pTrigger;

    	Partical* pNext;		// A partical pointer to the next partical a ParticalCollection
	    Partical* pPrev;		// A partical pointer to the next previous a ParticalCollection

        Partical ();
        Partical (SDL_Surface* pSrcBitmap, int x, int y, int w, int h);

        void blank ();

		void releaseBitmap (void);

        virtual ~Partical ();

        bool show (SDL_Surface* pScreen);

        bool move (void);

		void setTrigger (ParticalTrig* pTrigData);

        bool assignImage (SDL_Surface* pSrcBitmap, int x, int y, int w, int h);
};

// -===================================================================-
enum PCExplodeType {PC_VERT, PC_HORZ, PC_DIAGLEFT, PC_DIAGRIGHT, PC_EXPOUT};

#define ALLOCSIZE (512 * 20)

#define PARTLIST_CACHE_ENABLED 

class ParticalCollection
{
    public:
        Partical*           pList;		
        ParticalCollection* pCacheList;
        int                 listNo;

        ParticalCollection ()
        {            
            pList      = NULL;
            listNo     = 0;
            pCacheList = NULL;
        }

        virtual ~ParticalCollection ();
      
        Partical* removeItem (Partical*& pNode)
        {
            if (pNode == NULL)
                return NULL;
            
            if (pNode->pNext != NULL)
                pNode->pNext->pPrev = pNode->pPrev;
    
            if (pNode->pPrev != NULL)
                pNode->pPrev->pNext = pNode->pNext;

            Partical* pTmp = pNode->pNext;
            Partical* pRemovedItem = pNode;
            
            if (pNode == pList)
                pList = pTmp;

            pNode = pTmp;
	        listNo--;
	        return pRemovedItem;             
        }
        
        void delItem (Partical*& pNode)
        {
            Partical* i;  

            if ((i = removeItem (pNode)) != NULL)
            {
	            delete i;
            }
        }

        void cacheItem (Partical*& pNode)
        {
            if (pCacheList == NULL)            
                pCacheList = new ParticalCollection();

            pCacheList->addItem(removeItem (pNode));
        }

        Partical* findFreeItem (void);
       
        Partical* addItem (Partical* pNode);

        Partical* addNewItem (SDL_Surface* pSrcBitmap, int x, int y, int w, int h)
        {
            Partical* pFreeItem = findFreeItem ();

            if (pFreeItem != NULL)
            {
                pFreeItem->assignImage (pSrcBitmap, x, y, w, h);
                return addItem (pFreeItem);
            }

            return addItem (new Partical (pSrcBitmap, x, y, w, h));
        }

        // collection movers/showers
        void moveShowAll (SDL_Surface* pScreen)
        {
            for (Partical* pNode = pList; pNode != NULL;)
            {
                if (pNode->life != 0)
                {
					pNode->move();
					pNode->show(pScreen);
                }
                else
                {
#ifdef PARTLIST_CACHE_ENABLED
                    cacheItem (pNode);
#else
                    delItem  (pNode);                    
#endif
                    continue;
                }

                if (pNode != NULL)
                    pNode = pNode->pNext;
            }
        }

		void deleteAll (void)
        {
            for (Partical* pNode = pList; pNode != NULL;)
            {
				delItem(pNode);
            }
        }

        void showAll (SDL_Surface* pScreen)
        {
            for (Partical* pNode = pList; pNode != NULL;)
            {
				pNode->show(pScreen);

                if (pNode != NULL)
                    pNode = pNode->pNext;
            }
        }

		// ------------------------------------------------------------------------------------------------

        // type :
        // 0 = vertical
        // 1 = horizontal
        // 2 = diagonal left
		// 3 = diagonal right
		// 4 = explode out 
        bool buildExplosionSplit (SDLSprite* pSprite, PCExplodeType type, bool implode = false, float speedDelta = 1.5, int time2Live = 50, int blkSize = PARTBLKSIZE, ParticalTrig* pTrigData = NULL);
};

#endif
