#include <stdlib.h>
#include "player.h"
#include "level_design.h"
#include "walker_objects.h"

Player::~Player()
{
}

Player::Player ()
{
    // Setup main character sprite
    Create (GameBMResource[GR_PLAYER].filename,  GameBMResource[GR_PLAYER].frameWidth, GameBMResource[GR_PLAYER].frameHeight, GameBMResource[GR_PLAYER].frameNum);
                            
    SetColorKey(GameBMResource[GR_PLAYER].ckr, GameBMResource[GR_PLAYER].ckg,GameBMResource[GR_PLAYER].ckb);                       
    m_FrameDelay = 4; // Setup animation settings...

    levelReset ();
	playersScore = 0;		
    gameLevel    = 0;
	lastMove     = OBJT_NULL;
    m_VelX       = 2.6f;
    m_VelY       = 2.6f;
	playerLives  = 3;
	SetYFrameBank (0);
}

void Player::incrementBonusMultiplier (void)
{
	if (bonusMultipler < 5)
		bonusMultipler++;
}

void Player::setPlayerPOW (PLYPOWType type)
{
    int pt = (GameFPS.rate * 4); // 4 seconds of death.

    // Increase that time based upon bonus multiplier, to a max 2.5 seconds
    if (bonusMultipler > 0)
        pt += ((GameFPS.rate/5) * bonusMultipler);            

    switch (type)
    {
        case PPOW_HOTBULLETS:
            hotBulletTime = pt;
            break;

		case PPOW_SHOCKBLASTER:
			powShockBlasterTime = 0;
        case PPOW_3WAYFIRE:
        case PPOW_FASTERFIRE:			
            powTime = pt;
            powType = type;			
            break;

		case PPOW_ATTRACT:
            powTime = pt;
            powType = type;			
			setAttractMode (true);
			break;

        case PPOW_NONE:
            powTime = 0;
            powType = type;
			setAttractMode (false);
            break;
    }
}

void Player::setLevelStartPos (void)
{
    SetPos((float)(GamePlayArea.xC - m_BlockWidth/2), (float)(GamePlayArea.yC - m_BlockHeight/2));
}

void Player::add2Score (int amnt)
{
	int oldLife = (playersScore+1) / EXTRA_LIFE; // calc extra life ...
	playersScore += amnt;

	if (playersScore < 0)
		playersScore = 0;

	if ((playersScore / EXTRA_LIFE) > oldLife)
	{
		playerLives++;
		Mix_PlayChannel(SNDCHL_EXTRALIFE, GameSNDResource[GRS_OOW].pResSnd, 0);
	}
}

void Player::levelReset (void)
{
    setLevelStartPos();

	bonusMultipler		= 0;
    hotBulletTime		= 0;
    m_HitPoints			= 1;
    powType				= PPOW_NONE;
    powTime				= 0;
	powShockBlasterTime = 0;
	pShockBlasterObj	= NULL;
    gotHit				= false;
}

void Player::PlaySnd (SoundType sndID) 
{
	switch (sndID)
	{
        case SNDT_INTRO:
            Mix_PlayChannel(SNDCHL_PLAYER, GameSNDResource[GRS_PLAYERUP].pResSnd, 0);
            break;

		case SNDT_SHOOT:
			SNDPlayPan (SNDCHL_PLAYER, GameSNDResource[GRS_PLAYER_SHOOT].pResSnd, 0, (int)m_PosX+(m_BlockWidth/2), (int)m_PosY+(m_BlockHeight/2));
			break;

		case SNDT_DEATH:
            Mix_HaltChannel (-1);
			Mix_PlayChannel (SNDCHL_PLAYER, GameSNDResource[GRS_PLAYER_DEATH].pResSnd, 0);
			break;
	}		
}

void Player::setAttractMode (bool onOff)
{
    // Find the farest enemy to target first
	for (SDLSprite* pNode = PFriendsList->m_pList; pNode != NULL;)
	{         
        if ((pNode->m_Type >= OBJT_WKO) && (pNode->m_Type < OBJT_PLY))
		{
            ((WalkerObject*)pNode)->setAttartMode (onOff);
		}

        if (pNode != NULL)
            pNode = pNode->m_Next;
    }
}

SDLSprite* Player::getShockBlasterObj (int& distance)
{
    SDLSprite* pTarget = NULL;
    float      distX   = 0;
    float      distY   = 0;
    float      chkDistX, chkDistY;

    // Find the farest enemy to target first
	for (SDLSprite* pNode = PEnemyList->m_pList; pNode != NULL;)
	{                      
		if (pNode->m_TimeToLive == -1)
		{
			chkDistX = abs(pNode->m_PosX - m_PosX);
			chkDistY = abs(pNode->m_PosY - m_PosY);                    
			if ((chkDistX+chkDistY > distX + distY))
			{
				distX = chkDistX;
				distY = chkDistY;
				pTarget = pNode;
			}                    
		}

        if (pNode != NULL)
            pNode = pNode->m_Next;
    }

	distance = (int)(distX + distY);
	return pTarget;
}

void Player::processKeyboard (void)
{
    // Check player movements
	ObjectType thisMove = OBJT_NULL;

    if (GameKeyboard[GameInfo.userKeys[USRKEY_DOWN]] != false)
    {
		thisMove = PLY_BULLET_LL; 
        m_PosY += m_VelY;

        switch (GetYFrameBank())
        {
            case 0:
                SetYFrameBank (2);
                break;

            case 1:
		        SetYFrameBank (3);
                break;
        }
    }

    if (GameKeyboard[GameInfo.userKeys[USRKEY_UP]] != false)
    {
		thisMove = PLY_BULLET_UU;                
        m_PosY -= m_VelY;

        switch (GetYFrameBank())
        {
            case 0:
                SetYFrameBank (2);
                break;

           case 1:
		        SetYFrameBank (3);
                break;
        }
    }

    if (GameKeyboard[GameInfo.userKeys[USRKEY_LEFT]] != false)
    {
		thisMove = PLY_BULLET_LL;
        m_PosX -= m_VelX;
        SetYFrameBank (3);
    }

    if (GameKeyboard[GameInfo.userKeys[USRKEY_RIGHT]] != false)
    {
		thisMove = PLY_BULLET_RR;
        m_PosX += m_VelX;
        SetYFrameBank (2);
    }

	// change frame to standing pose if no key input from user
    if ((lastMove != OBJT_NULL) && (thisMove == OBJT_NULL))
	{				 
        switch (GetYFrameBank())
        {
            case 3:
                SetYFrameBank (1);
                break;

           case 2:
		        SetYFrameBank (0);
                break;
        }
		lastMove = OBJT_NULL;
	}
	else if (thisMove != OBJT_NULL)
	{
        GameBoundryCheckUpdate (this);

		lastMove = thisMove;
	}			

	AnimateSprite (GameFPS.totalframecount);

    // How quickly to shot, normal & power-up
    if ( ((GameFPS.totalframecount % 10 == 0) || ((powType == PPOW_FASTERFIRE) && (GameFPS.totalframecount % 6) == 0)) )
    {
        SDLSprite* pNewBullet[] = {NULL,NULL,NULL};
        ObjectType type[3];
        float      xSpd[3];
        float      ySpd[3];

        // Fire right & Up
        if ((GameKeyboard[GameInfo.userKeys[USRKEY_FIREUP]] != false) && (GameKeyboard[GameInfo.userKeys[USRKEY_FIRERIGHT]] != false))
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET3].pResObj);
            type[0]       = PLY_BULLET_RU;
            xSpd[0]       = 8;
            ySpd[0]       = -8;
        }
        // Fire left & down
        else if ((GameKeyboard[GameInfo.userKeys[USRKEY_FIREDOWN]] != false) && (GameKeyboard[GameInfo.userKeys[USRKEY_FIRELEFT]] != false))
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET3].pResObj);
            type[0]       = PLY_BULLET_LD;
            xSpd[0]       = -8;
            ySpd[0]       = 8;
        }
        // Fire right & down
        else if ((GameKeyboard[GameInfo.userKeys[USRKEY_FIREDOWN]] != false) && (GameKeyboard[GameInfo.userKeys[USRKEY_FIRERIGHT]] != false))
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET2].pResObj);
            type[0]       = PLY_BULLET_RD;
            xSpd[0]       = 8;
            ySpd[0]       = 8;
        }
        // Fire left & up
        else if ((GameKeyboard[GameInfo.userKeys[USRKEY_FIREUP]] != false) && (GameKeyboard[GameInfo.userKeys[USRKEY_FIRELEFT]] != false))
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET2].pResObj);
            type[0]       = PLY_BULLET_LU;
            xSpd[0]       = -8;
            ySpd[0]       = -8;
        }
        // Fire right
        else if (GameKeyboard[GameInfo.userKeys[USRKEY_FIRERIGHT]] != false)
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET0].pResObj);
            type[0]       = PLY_BULLET_RR;
            xSpd[0]       = 8;
            ySpd[0]       = 0;
        }
        // Fire left
        else if (GameKeyboard[GameInfo.userKeys[USRKEY_FIRELEFT]] != false)
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET0].pResObj);
            type[0]       = PLY_BULLET_LL;
            xSpd[0]       = -8;
            ySpd[0]       = 0;
        }
        // Fire up
        else if (GameKeyboard[GameInfo.userKeys[USRKEY_FIREUP]] != false)
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET1].pResObj);
            type[0]       = PLY_BULLET_UU;
            xSpd[0]       = 0;
            ySpd[0]       = -8;
        }
        // Fire down
        else if (GameKeyboard[GameInfo.userKeys[USRKEY_FIREDOWN]] != false)
        {
            pNewBullet[0] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET1].pResObj);
            type[0]       = PLY_BULLET_DD;
            xSpd[0]       = 0;
            ySpd[0]       = 8;
        }

        if (pNewBullet[0] != NULL)
        {
			PlaySnd (SNDT_SHOOT); 					

            switch (powType)
            {
                case PPOW_3WAYFIRE:
                    switch (type[0])
                    {
                        case PLY_BULLET_LL:
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET2].pResObj);
                            type[1]       = PLY_BULLET_LU;
                            xSpd[1]       = -8;
                            ySpd[1]       = -8;
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET3].pResObj);
                            type[2]       = PLY_BULLET_LD;
                            xSpd[2]       = -8;
                            ySpd[2]       = 8;
                            break;
                        case PLY_BULLET_LU:
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET0].pResObj);
                            type[1]       = PLY_BULLET_LL;
                            xSpd[1]       = -8;
                            ySpd[1]       = 0;
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET1].pResObj);
                            type[2]       = PLY_BULLET_UU;
                            xSpd[2]       = 0;
                            ySpd[2]       = -8;
                            break;
                        case PLY_BULLET_UU:
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET2].pResObj);
                            type[1]       = PLY_BULLET_LU;
                            xSpd[1]       = -8;
                            ySpd[1]       = -8;
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET3].pResObj);
                            type[2]       = PLY_BULLET_RU;
                            xSpd[2]       = 8;
                            ySpd[2]       = -8;
                            break;
                        case PLY_BULLET_RU:
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET1].pResObj);
                            type[1]       = PLY_BULLET_UU;
                            xSpd[1]       = 0;
                            ySpd[1]       = -8;
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET0].pResObj);
                            type[2]       = PLY_BULLET_RR;
                            xSpd[2]       = 8;
                            ySpd[2]       = 0;
                            break;
                        case PLY_BULLET_RR:
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET3].pResObj);
                            type[1]       = PLY_BULLET_RU;
                            xSpd[1]       = 8;
                            ySpd[1]       = -8;
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET2].pResObj);
                            type[2]       = PLY_BULLET_RD;
                            xSpd[2]       = 8;
                            ySpd[2]       = 8;
                            break;
                        case PLY_BULLET_RD:
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET0].pResObj);
                            type[1]       = PLY_BULLET_RR;
                            xSpd[1]       = 8;
                            ySpd[1]       = 0;
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET1].pResObj);
                            type[2]       = PLY_BULLET_DD;
                            xSpd[2]       = 0;
                            ySpd[2]       = 8;
                            break;
                        case PLY_BULLET_DD:    
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET2].pResObj);
                            type[1]       = PLY_BULLET_RD;
                            xSpd[1]       = 8;
                            ySpd[1]       = 8;                                    
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET3].pResObj);
                            type[2]       = PLY_BULLET_LD;
                            xSpd[2]       = -8;
                            ySpd[2]       = 8;
                            break;
                        case PLY_BULLET_LD:          
                            pNewBullet[1] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET1].pResObj);
                            type[1]       = PLY_BULLET_DD;
                            xSpd[1]       = 0;
                            ySpd[1]       = 8; 
                            pNewBullet[2] = new SDLSprite (GameBMResource[GR_PLAYER_BULLET0].pResObj);
                            type[2]       = PLY_BULLET_LL;
                            xSpd[2]       = -8;
                            ySpd[2]       = 0;                                    
                            break;
                    }
                    break;
            }

            for (int idx = 0; idx < 3; idx++)
            {
                if (pNewBullet[idx] == NULL)
                    break;

                PPlayerBulletList->AddSpriteVals(pNewBullet[idx], type[idx],  
                                                    m_PosX + (m_BlockWidth/2),
                                                    m_PosY + (m_BlockHeight/2),
                                                    xSpd[idx], ySpd[idx]);
                                
                if (hotBulletTime > 0) // setup frame type and hit count amount
                {
                    pNewBullet[idx]->SetYFrameBank(1);
                    pNewBullet[idx]->m_HitPoints = 5;
                }
                else
				{
					pNewBullet[idx]->SetYFrameBank(0);
                    pNewBullet[idx]->m_HitPoints = 1;
				}

                pNewBullet[idx]->m_IsBullet = SDL_TRUE;
            }
        }
    }

	// Decrement power up time ...
	if (hotBulletTime > 0)
		hotBulletTime--;

	if (powTime > 0)
	{
		powTime--;
		if (powTime <= 0)
			setPlayerPOW (PPOW_NONE);
	}

	if (powType == PPOW_SHOCKBLASTER)
	{
		bool safeObj = false;

		if ((pShockBlasterObj != NULL) && (powShockBlasterTime > 0))
		{
			// check if pLockedObj is safe to use?		
			for (SDLSprite* pNode = PEnemyList->m_pList; pNode != NULL;)
			{
				if (pNode == pShockBlasterObj)
				{
					safeObj = true;
					break;
				}

                if (pNode != NULL)
                    pNode = pNode->m_Next;
			}
		}

		if (safeObj == false)
		{
			if ((pShockBlasterObj = getShockBlasterObj (shockDist)) != NULL)
			{
				powShockBlasterTime = GameFPS.rate/4;
                SNDPlayPan(SNDCHL_POWSHOCKBLAST, GameSNDResource[GRS_ELECTRIC1].pResSnd, 0, (int)pShockBlasterObj->m_PosX, (int)pShockBlasterObj->m_PosY);				
			}
		}		

		if (pShockBlasterObj != NULL)									
		{
			powShockBlasterTime--;
			if (powShockBlasterTime > 0)
			{								
				// Draw Electric line from player to object.
				SDL_Lightning (PScreen, 
							   (int)pShockBlasterObj->m_PosX + (pShockBlasterObj->m_BlockWidth/2),
							   (int)pShockBlasterObj->m_PosY + (pShockBlasterObj->m_BlockHeight/2),
							   (int)m_PosX + (m_BlockWidth/2),
							   (int)m_PosY + (m_BlockHeight/2),
							   10, 16);
			}
			else
			{
                pShockBlasterObj->m_HitPoints -= 10; // same hitpoints as a hot button.
                if (pShockBlasterObj->m_HitPoints <= 0)
                {
				    pShockBlasterObj -> PlaySnd (SNDT_DEATH);
				    CheckSpawnCreateOnDeath (pShockBlasterObj);

					add2Score (pShockBlasterObj->m_Score); 
            
				    PExpObjs->buildExplosionSplit (pShockBlasterObj, (PCExplodeType)(rand() % 5));            
				    PEnemyList->DelSprite(pShockBlasterObj);
				    pShockBlasterObj = NULL;
                }
			}
		}
	}
}
