#include <stdlib.h>
#include "walker_objects.h"
#include "level_design.h"


WalkerObject::WalkerObject (ObjectType t)
{        
	pHomer    = NULL;
	homerInit = false;


  	dirSpriteIdx  = 0;
	dirChgTimer   = 0;
	direction     = 0;
    powBlinkTime  = 0;
	attractMode   = false;

    switch (t)
    {
		case WKO_POWATTRACT:
            Create (GameBMResource[GR_POW_ATTRACTOR].pResObj);
			m_HitPoints  = GameBMResource[GR_POW_ATTRACTOR].hitPoints;
			m_Score      = GameBMResource[GR_POW_ATTRACTOR].score;
            m_FrameDelay = 4;
			m_TimeToLive = (GameFPS.rate * 7);            
            powBlinkTime = (GameFPS.rate * 3);
            m_VelX       = 0.5f;
            m_VelY       = 0.5f;
			break;

        case WKO_POWSHOCKBLASTER:
            Create (GameBMResource[GR_POW_SHOCKBLASTER].pResObj);
			m_HitPoints  = GameBMResource[GR_POW_SHOCKBLASTER].hitPoints;
			m_Score      = GameBMResource[GR_POW_SHOCKBLASTER].score;
            m_FrameDelay = 4;
			m_TimeToLive = (GameFPS.rate * 7);            
            powBlinkTime = (GameFPS.rate * 3);
            m_VelX       = 0.5f;
            m_VelY       = 0.5f;
            break;

        case WKO_POWSMARTBOMB:
            Create (GameBMResource[GR_POW_SMARTBOMB].pResObj);
			m_HitPoints  = GameBMResource[GR_POW_SMARTBOMB].hitPoints;
			m_Score      = GameBMResource[GR_POW_SMARTBOMB].score;
            m_FrameDelay = 4;
			m_TimeToLive = (GameFPS.rate * 7);
            powBlinkTime = (GameFPS.rate * 3);
            m_VelX       = 0.5f;
            m_VelY       = 0.5f;
            break;

        case WKO_POW3FIRE:
            Create (GameBMResource[GR_POW_3FIRE].pResObj);
			m_HitPoints  = GameBMResource[GR_POW_3FIRE].hitPoints;
			m_Score      = GameBMResource[GR_POW_3FIRE].score;
            m_FrameDelay = 4;
			m_TimeToLive = (GameFPS.rate * 7);
            powBlinkTime = (GameFPS.rate * 3);
            m_VelX       = 0.5f;
            m_VelY       = 0.5f;
            break;

        case WKO_POWFASTERFIRE:
            Create (GameBMResource[GR_POW_FASTERFIRE].pResObj);
			m_HitPoints  = GameBMResource[GR_POW_FASTERFIRE].hitPoints;
			m_Score      = GameBMResource[GR_POW_FASTERFIRE].score;
            m_FrameDelay = 4;
			m_TimeToLive = (GameFPS.rate * 7);
            powBlinkTime = (GameFPS.rate * 3);
            m_VelX       = 0.5f;
            m_VelY       = 0.5f;
            break;

        case WKO_POWEXTRALIFE:
            Create (GameBMResource[GR_POW_EXTRALIFE].pResObj);
			m_HitPoints  = GameBMResource[GR_POW_EXTRALIFE].hitPoints;
			m_Score      = GameBMResource[GR_POW_EXTRALIFE].score;
            m_FrameDelay = 4;
			m_TimeToLive = (GameFPS.rate * 7);
            powBlinkTime = (GameFPS.rate * 3);
            m_VelX       = 0.5f;
            m_VelY       = 0.5f;
            break;

		case WKO_ROBBYZOMBIE:
			{			
                pHomer = new DynamicEnemy (DET_ROBBYZOMBIE1);

				Create (GameBMResource[GR_ROBBY].pResObj);
				m_HitPoints  = GameBMResource[GR_ROBBY].hitPoints;
				m_Score      = GameBMResource[GR_ROBBY].score;
				m_IsBullet   = SDL_TRUE;
                dirSpriteIdx = 1;
				SetYFrameBank (dirSpriteIdx);
      		}
			break;

		case WKO_RAIN1:
			{
                Create (GameBMResource[GR_FRD_RAIN1].pResObj);
				m_HitPoints  = GameBMResource[GR_FRD_RAIN1].hitPoints;
				m_Score      = GameBMResource[GR_FRD_RAIN1].score;
                m_FrameDelay = 10;
			}
			break;

		case WKO_WENCH:
			{
                Create (GameBMResource[GR_WENCH].pResObj);
				m_HitPoints  = GameBMResource[GR_WENCH].hitPoints;
				m_Score      = GameBMResource[GR_WENCH].score;
                m_FrameDelay = 10;
			}
			break;

        case WKO_MAX:
			{		
                Create (GameBMResource[GR_MAX].pResObj);
				m_HitPoints  = GameBMResource[GR_MAX].hitPoints;
				m_Score      = GameBMResource[GR_MAX].score;
                m_FrameDelay = 4;
			}
            break;

		case WKO_SAM:
			{
                Create (GameBMResource[GR_SAM].pResObj);
				m_HitPoints  = GameBMResource[GR_SAM].hitPoints;
				m_Score      = GameBMResource[GR_SAM].score;
                m_FrameDelay = 10;
			}
			break;
    }
    m_Type = t;
}


void WalkerObject::PlaySnd (int sndID)
{
	switch (sndID)
	{
		case SNDT_HIT:
        case SNDT_DEATH:
			switch (m_Type)
			{
                case WKO_WENCH:
				case WKO_SAM:
				case WKO_MAX:
					SNDPlayPan (SNDCHL_PICKFRIEND, GameSNDResource[GRS_PICKUP3].pResSnd, 0, (int)m_PosX, (int)m_PosY); 
					break;

				case WKO_POWEXTRALIFE:
					SNDPlayPan (SNDCHL_EXTRALIFE, GameSNDResource[GRS_OOW].pResSnd, 0, (int)m_PosX, (int)m_PosY); 
					break;

                case WKO_POWSMARTBOMB:
                    Mix_PlayChannel (SNDCHL_SMARTBOMB, GameSNDResource[GRS_SMARTBOMB1].pResSnd, 0); 
                    break;                    

				case WKO_POWATTRACT:
                case WKO_POWSHOCKBLASTER:
				case WKO_POWFASTERFIRE:
				case WKO_POW3FIRE:				
					SNDPlayPan (SNDCHL_POW, GameSNDResource[GRS_PICKUP1].pResSnd, 0, (int)m_PosX, (int)m_PosY);                        
					break;

                case WKO_ROBBYZOMBIE:
                    SNDPlayPan (SNDCHL_ENEMY_HIT, GameSNDResource[GRS_ENEMYHIT1].pResSnd, 0, (int)m_PosX, (int)m_PosY);
                    break;
			}
			break;
	}
}

WalkerObject::~WalkerObject ()
{
    if (pHomer != NULL)
        delete pHomer;
}

void WalkerObject::Draw(SDL_Surface* lpSDLS, long int ScrnWorldX , long int ScrnWorldY)
{
	switch (m_Type)
	{
		case WKO_ROBBYZOMBIE:
			{
				for (int idx = 0; idx < pHomer->movePntIdx; idx++)
				{	
					if (pHomer->movePnts[idx].velX > 0.0f)      // moving right...
                        dirSpriteIdx = 2;
					else if (pHomer->movePnts[idx].velX < 0.0f) // moving left...
                        dirSpriteIdx = 0;
					else if (pHomer->movePnts[idx].velY > 0.0f) // moving down...
                        dirSpriteIdx = 3;
					else if (pHomer->movePnts[idx].velY < 0.0f) // moving up...
                        dirSpriteIdx = 1;


					SetYFrameBank (dirSpriteIdx);
					m_PosX = pHomer->movePnts[idx].x - m_BlockWidth/2;
					m_PosY = pHomer->movePnts[idx].y - m_BlockHeight/2;
                    AnimateSprite(GameFPS.totalframecount);
					SDLSprite::Draw (PScreen);
				}
			}
			break;

		default:
            // Blink POW type objects that are about to expire
            if (powBlinkTime > 0)
            {
                if (m_TimeToLive <= (powBlinkTime/2))
                {
                    if ((GameFPS.totalframecount % 3) == 0)
                        SDLSprite::Draw (lpSDLS, ScrnWorldX, ScrnWorldY);                    
                }
                else if (m_TimeToLive <= powBlinkTime)
                {
                    if ((GameFPS.totalframecount % 4) == 0)
                        SDLSprite::Draw (lpSDLS, ScrnWorldX, ScrnWorldY);                    
                }
                else 
                     SDLSprite::Draw (lpSDLS, ScrnWorldX, ScrnWorldY);                    
            }
            else
                SDLSprite::Draw (lpSDLS, ScrnWorldX, ScrnWorldY);
	}

	// Display attract mode magnet
	if (attractMode != false)
	{
		switch (m_Type)
		{
			case WKO_SAM:
			case WKO_MAX:
            case WKO_WENCH:
				SDLSprite* pMag = GameBMResource[GR_POW_ATTRACTMAG].pResObj;
				pMag->m_PosX = (m_PosX + m_BlockWidth/2) - (pMag->m_BlockWidth/2);
				pMag->m_PosY = m_PosY - pMag->m_BlockHeight;
				pMag->Draw (lpSDLS, ScrnWorldX, ScrnWorldY);
				break;
		}
	}
}

void WalkerObject::setAttartMode (bool onOff)
{
	switch (m_Type)
	{
		case WKO_SAM:
		case WKO_MAX:
        case WKO_WENCH:
			attractMode = onOff;
			break;
	}
}

void WalkerObject::Move (SDLSprite* pPlayer)
{
	switch (m_Type)
	{
        case WKO_RAIN1:
            // Rebound off the walls of the arena
            if ((int)m_PosY+m_BlockHeight+m_VelY > GamePlayArea.y2)
                m_VelY = -m_VelY;
            if ((int)m_PosX+m_VelX < GamePlayArea.x1)
                m_VelX = -m_VelX;
            if ((int)m_PosY+m_VelY < GamePlayArea.y1)
                 m_VelY = -m_VelY;
            if ((int)m_PosX+m_BlockWidth+m_VelX > GamePlayArea.x2)
                m_VelX = -m_VelX;

            m_PosX += m_VelX;
			m_PosY += m_VelY;	

            if ((rand() % 5) == 2)
            {
                SDLSprite* pNewObj = CreateNewObject (BET_SHOT_REFIRE, LLT_NULL);

                pNewObj->m_PosX = (float)(GamePlayArea.x1 + (rand() % (GamePlayArea.w  * 2)));

                if (pNewObj->m_PosX > GamePlayArea.w)
                {
                    pNewObj->m_PosY = (float)(GamePlayArea.y1 + (pNewObj->m_PosX - GamePlayArea.w));
                    pNewObj->m_PosX = (float)(GamePlayArea.w-10);
                }
                else
                    pNewObj->m_PosY = (float)(GamePlayArea.y1 + 0);

                pNewObj->m_VelX      = -1.5f;
                pNewObj->m_VelY      = 1.5f;
                pNewObj->m_HitPoints = 1;
                pNewObj->SetYFrameBank (5);
                PEnemyList->AddSprite (pNewObj);
            }
            AnimateSprite(GameFPS.totalframecount);
            break;


		case WKO_ROBBYZOMBIE:
			if (homerInit == false)
			{
				pHomer->setUpHomer (m_PosX, m_PosY);
				homerInit = true;
			}
			pHomer->Move (pPlayer);
            break;

		case WKO_POWATTRACT:
        case WKO_POWSHOCKBLASTER:
        case WKO_POWSMARTBOMB:
        case WKO_POWFASTERFIRE:
        case WKO_POW3FIRE:
        case WKO_POWEXTRALIFE:
		case WKO_SAM:
		case WKO_MAX:
        case WKO_WENCH:

			dirChgTimer--;
			if (dirChgTimer <= 0)
			{
				int d;
				int di = 0;

				if (attractMode == false) 
				{
					d			= rand() % 5;					
					dirChgTimer = 20 + rand() % 50;
					switch (d)
					{
						case 1: // left
							di = 0;							
							break;

						case 2: // up 
							di = 1;
							break;

						case 3: // right 
							di = 2;
							break;

						case 4: // down
							di = 3;
							break;
						
						case 0:
						default: // stop
							if (direction == 2)
								di = 1;
							else
								di = 3;
					}
				}
				else // move towards the player, and faster
				{				
					if (m_PosX -(m_VelX * 3)+(m_BlockWidth/2) > pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
					{
						m_PosX = m_PosX - (m_VelX * 3);
						di = 0;
						d  = 1;
					}
					else if (m_PosX + (m_VelX * 3)+(m_BlockWidth/2) < pPlayer->m_PosX + (pPlayer->m_BlockWidth/2))
					{
						m_PosX = m_PosX + (m_VelX * 3);
						di = 2;
					}

					if (m_PosY - (m_VelY * 3) + (m_BlockHeight/2) > pPlayer->m_PosY + (pPlayer->m_BlockHeight/2) )
					{
						m_PosY = m_PosY - (m_VelY * 3);
						di = 1;
						d  = 2;
					}
					else if (m_PosY + (m_VelY * 3) + (m_BlockHeight/2) < pPlayer->m_PosY + (pPlayer->m_BlockHeight/2))
					{
						m_PosY = m_PosY + (m_VelY * 3);
						di = 3;
						d  = 4;
					}
				}

				// set animation frame bank
				if (direction != d)
				{
					direction    = d;
					dirSpriteIdx = di;
					
                    // Change direction sprite on these object types...
                    switch (m_Type)
                    {
		                case WKO_SAM:
		                case WKO_MAX:
                        case WKO_WENCH:
					        SetYFrameBank (di);
                            m_Frame      = 0;
                    }					
				}
			}	
			else
			{
				switch (direction)
				{
					case 1: // left
						m_PosX -= m_VelX;
						break;

					case 2: // up 
						m_PosY -= m_VelY;
						break;

					case 3: // right 
						m_PosX += m_VelX;
						break;

					case 4: // down
						m_PosY += m_VelX;
						break;
					
					case 0:
					default: // stop
						;
				}

			}

            switch (m_Type)
            {
		        case WKO_SAM:
		        case WKO_MAX:
                case WKO_WENCH:
                    // Dont animate stop movement
                    if (direction != 0)
                        AnimateSprite(GameFPS.totalframecount);
                    break;

                default:
                    AnimateSprite(GameFPS.totalframecount);
            }	            
			break;
	}

    GameBoundryCheckUpdate (this);
}